# Copyright(c) 2002-2003 Association of Universities for Research in Astronomy, Inc.

procedure gemarith(operand1, op, operand2, result)

# Arithmetic on MEF files, replaces gmos.garith
#
# Version  Sept 10, 2002 IJ  Release v1.4
#          Nov 20, 2002  IJ  Give error msg for verbose-

string  operand1  {prompt="Operand MEF image"}
string  op        {"",prompt="Operator", enum="+|-|*|/|min|max"}
string  operand2  {prompt="Operand MEF image or numerical constant"}
string  result    {prompt="Output MEF file"}
string  sci_ext   {"SCI",prompt="Name of science extension"}
string  var_ext   {"VAR",prompt="Name of variance extension"}
string  dq_ext    {"DQ ",prompt="Name of data extension"}
bool    fl_vardq  {no,prompt="Propagate variance and data quality"}
string  logfile   {"gemarith.log",prompt="Logfile"}
bool    verbose   {no,prompt="Verbose?"}
int     status    {0,prompt="Exit status (0=good)"}

begin

string   l_operand1, l_operand2, l_op, l_result, l_sci_ext,l_var_ext,l_dq_ext
string  l_logfile
bool    l_fl_vardq, l_verbose

string  imga, imgb, seca, secb, wmeffirst
string  scia[100], vara[100], dqa[100], scib[100], varb[100], dqb[100]
string  tmp, img, sciexta, sciextb,varexta, varextb, dqexta, dqextb,dqexpr
int  ii, jj, kk, numext, n_ext
bool  itsanimageA, itsanimageB, fl_foundmdf
struct sdate

# Make temp file
tmp = mktemp("tmp")

status =0
cache("gemlogname")

# Query parameters
l_operand1 = operand1 ; l_op = op; l_operand2 = operand2
l_result = result; l_fl_vardq=fl_vardq; l_verbose=verbose; l_logfile=logfile
l_sci_ext=sci_ext;l_var_ext=var_ext;l_dq_ext=dq_ext

gemlogname(logpar=l_logfile, package="")
l_logfile = gemlogname.logname

date | scan(sdate)
printlog("-------------------------------------------------------------\
-------------------",l_logfile,l_verbose)
printlog("GEMARITH -- "//sdate,l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)
printlog("operand1 = "//l_operand1,l_logfile,l_verbose)
printlog("op       = "//l_op,l_logfile,l_verbose)
printlog("operand2 = "//l_operand2,l_logfile,l_verbose)
printlog("result   = "//l_result, l_logfile,l_verbose)
printlog("sci_ext  = "//l_sci_ext,l_logfile,l_verbose)
printlog("var_ext  = "//l_var_ext,l_logfile,l_verbose)
printlog("dq_ext   = "//l_dq_ext,l_logfile,l_verbose)
printlog("fl_vardq = "//l_fl_vardq,l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)

# Check that the output image does not exist - exit otherwise
gimverify(l_result); l_result=gimverify.outname
if(gimverify.status==0 || gimverify.status==4) {
     printlog("ERROR - GEMARITH: Output image "//l_result//" exists",l_logfile,yes)
     status=1
     goto clean
}     

# Figure out if operand1 and 2 are images. Start with operand 1:
itsanimageA=no; imga = l_operand1
gimverify(imga); imga=gimverify.outname
if(gimverify.status==0) {
     itsanimageA=yes
# The PHU of the first image is used for the output image l_result
# Here the PHU of the first image is assigned to a variable  
     wmeffirst=gimverify.outname//"[0]"
}
# Now go on to operand 2:
itsanimageB=no; imgb = l_operand2
gimverify(imgb); imgb=gimverify.outname
if(gimverify.status==0) itsanimageB=yes

# This task will not work if both operands do not exist or both are numbers:
if(!itsanimageA && !itsanimageB) {
  printlog("ERROR - GEMARITH: Both inputs are numbers or the image(s) do not exist",l_logfile,yes)
  status=1
  goto clean
} 

# Get number of extensions. This depends on whether the inputs have been
# *prepared. If they have, then the sci_ext string is
# searched for in the output of fxheader. If they have not been run through
# *prepare, then each extension will be operated on.
# Note that the number of extensions on both inputs must be the same.
# Initialize some variables:
ii= 0; jj=0; kk=0
sciexta=""; varexta=""; dqexta=""
sciextb=""; varextb=""; dqextb=""
# Now start with first input:
if(itsanimageA){
  # Check if it's been prepared with nprepare or gprepare
        fxheader(l_operand1,format="",long+,count+)|match("PREPARE",
    meta-,stop-,print+) | count | scan(kk)
  if(kk==1) { # it has been prepared!
          fxheader(l_operand1,format="",long-,count+) | match(l_sci_ext,
      meta-,stop-,print+) | count | scan(ii)
    sciexta=l_sci_ext//",";varexta=l_var_ext//",";dqexta=l_dq_ext//","
  
  } else { # it has NOT been prepared.
          fxheader(l_operand1,format="",long-,count+, > tmp)
    tail(tmp,nlines=1) | scan(ii)
    delete(tmp,ver-)
    if(l_fl_vardq) {
      l_fl_vardq=no
      printlog("WARNING - GEMARITH: Raw input image, setting fl_vardq = no",
        l_logfile,l_verbose)
    }
  }  
  numext = ii
  kk=0
  if(l_fl_vardq) { 
    fxheader(l_operand1,format="",long-,count+) | match(l_var_ext,
      meta-,stop-,print+) | count | scan(kk)
    if(kk==0) {
      l_fl_vardq=no  
      printlog("WARNING - GEMARITH: No variance extensions in some input images, setting fl_vardq = no",
        l_logfile,l_verbose)
    } else {
      kk=0
      fxheader(l_operand1,format="",long-,count+) | match(l_dq_ext,
        meta-,stop-,print+) | count | scan(kk)
      if(kk==0) {
        l_fl_vardq=no      
        printlog("WARNING - GEMARITH: No DQ extensions in some input images, setting fl_vardq = no",l_logfile,
          l_verbose)
      }          
        
    }      
  }
  kk=0
}
# Go on to second input:
if(itsanimageB){
       fxheader(l_operand2,format="",long+,count+)|match("PREPARE",
    meta-,stop-,print+) | count | scan(kk)
  if(kk==1) {
          fxheader(l_operand2,format="",long-,count+) | match(l_sci_ext,
      meta-,stop-,print+) | count | scan(jj)
    sciextb=l_sci_ext//","; varextb=l_var_ext//","; dqextb=l_dq_ext//","
  
  } else {
          fxheader(l_operand2,format="",long-,count+, > tmp)
    tail(tmp,nlines=1) | scan(jj)
    delete(tmp,ver-)
    if(l_fl_vardq) {
      l_fl_vardq=no
      printlog("WARNING - GEMARITH: Raw input image, setting fl_vardq = no",
        l_logfile,l_verbose)
    }
  }  
  numext = jj
  
  if(l_fl_vardq) {
    fxheader(l_operand2,format="",long-,count+) | match(l_var_ext,
      meta-,stop-,print+) | count | scan(kk)
    if(kk==0) {
      l_fl_vardq=no
      printlog("WARNING - GEMARITH: No variance extensions in some input images, setting fl_vardq = no",l_logfile,
        l_verbose)
    } else {
      kk=0
      fxheader(l_operand2,format="",long-,count+) | match(l_dq_ext,
        meta-,stop-,print+) | count | scan(kk)
      if(kk==0) {
        l_fl_vardq=no
        printlog("WARNING - GEMARITH: No DQ extensions in some input images, setting fl_vardq = no",l_logfile,
          l_verbose)
      }          
    }      
  }    
      
}
l_var_ext=var_ext//",";l_dq_ext=dq_ext//","

# Check the number of extensions is the same in both images
if(itsanimageA && itsanimageB && ii!=jj) {
  printlog("ERROR - GEMARITH: Number of image extensions in \
    input images is different",l_logfile,l_verbose)
  status = 1
  goto clean
}

# Get the phu from the second image if the first input is a number:
if(!itsanimageA) wmeffirst=gimverify.outname//"[0]"
imcopy(wmeffirst,l_result//".fits",verbose-)

# If there is an MDF then preserve it - can only have MDF if NEXTEND
# is in the header
fl_foundmdf=no
if(itsanimageA) {
  keypar(imga//".fits[0]","NEXTEND",sile+, >>& "dev$null")
  if(keypar.found) {
    n_ext=int(keypar.value)
    for(ii=1;ii<=n_ext;ii+=1) {
      keypar(imga//".fits["//ii//"]","EXTNAME",sile+,>>& "dev$null")
      if(keypar.value=="MDF") {
        fl_foundmdf=yes
        fxinsert(imga//".fits["//ii//"]",l_result//".fits[0]",ii,verbose-, >>& "dev$null")
        ii=n_ext
      }
    }
  }
}
if(itsanimageB && !fl_foundmdf) {
  keypar(imgb//".fits[0]","NEXTEND",sile+, >>& "dev$null")
  if(keypar.found) {
    n_ext=int(keypar.value)
    for(ii=1;ii<=n_ext;ii+=1) {
      keypar(imgb//".fits["//ii//"]","EXTNAME",sile+,>>& "dev$null")
      if(keypar.value=="MDF") {
        fl_foundmdf=yes
        fxinsert(imgb//".fits["//ii//"]",l_result//".fits[0]",ii,verbose-, >>& "dev$null")
        ii=n_ext
      }
    }
  }
}
flpr


# Make the scib, varb, and dqb arrays with the correct extension and
# if applicable, the image section
# Start with input 2
secb=""
for(ii=1; ii<=numext; ii+=1) {
  scib[ii]=l_operand2
  if(itsanimageB) {
     if(stridx("[",l_operand2)>0 && stridx(":",l_operand2)>0) {
      imgb = substr(l_operand2,1,stridx("[",l_operand2)-1)//"["  
      secb = "]"//substr(l_operand2,stridx("[",l_operand2),strlen(l_operand2))
    } else if (stridx("[",l_operand2)==0) {
      imgb = l_operand2//"[" #imgb//"["//ii//"]"
      secb = "]"
    } else {
      printlog("ERROR - GEMARITH: Do not specify extension number in operand2",
        l_logfile,yes)
      status=1
    }
    scib[ii]=imgb//sciextb//ii//secb  
    varb[ii]=imgb//varextb//ii//secb
    dqb[ii]=","//imgb//dqextb//ii//secb
    
  } 
}  
# Go on to input 1
seca=""
for(ii=1; ii<=numext; ii+=1) {
  scia[ii]=l_operand1; vara[ii]=l_operand1; dqa[ii]=l_operand1
  if(itsanimageA) {
    if(stridx("[",l_operand1)>0 && stridx(":",l_operand1)>0) {
      imga = substr(l_operand1,1,stridx("[",l_operand1)-1)//"["
      seca = "]"//substr(l_operand1,stridx("[",l_operand1),strlen(l_operand1))
    } else if (stridx("[",l_operand1)==0) {
      imga = l_operand1//"[" #imga//"["//ii//"]"
      seca = "]"
    } else {
      printlog("ERROR - GEMARITH: Do not specify extension number in operand1",
        l_logfile,l_verbose)
      status=1
    }
    scia[ii]=imga//sciexta//ii//seca
    vara[ii]=imga//varexta//ii//seca
    dqa[ii]=imga//dqexta//ii//seca
  }
}


if(status==1) goto clean

# Operate on each extension - do the math on the science plane
for(ii=1; ii<=numext; ii+=1) {
  jj=0 ; delete(tmp,ver-, >>& "dev$null")
  imarith(scia[ii],l_op,scib[ii],
    l_result//"["//l_sci_ext//","//ii//",append]",title="",divzero=0.,
    hparams="",pixtype="",calctype="",verbose-,noact-, >& tmp)
# Trap error msg from imarith
    count(tmp) | scan(jj)
    if(jj>0) {
      type(tmp) | scan (sdate)
      printlog(sdate,l_logfile,yes)
      delete(l_result//".fits",ver-, >>& "dev$null")
      status=1
      goto clean
    }
}

if(!l_fl_vardq) goto clean

# Populate the vara and varb arrays with zeros and populate the dqa and dqb 
# arrays with empty strings if any of the inputs is a number.
dqexpr="im1"
if(itsanimageA && !itsanimageB) {
  for(ii=1; ii<=numext; ii+=1) {
    varb[ii] = 0.
    dqb[ii] = ""
  }
} else if (!itsanimageA && itsanimageB) {
  for(ii=1; ii<=numext; ii+=1) {
    vara[ii] = 0.
    dqa[ii] = ""
  }
}

# Propagate VAR/DQ
if(itsanimageA && itsanimageB) {
  dqexpr="im1 || im2"
}  
if(l_op=="+" || l_op=="-") {
  for(ii=1; ii<=numext; ii+=1) {
    imexpr("(a+b)",l_result//"["//l_var_ext//ii//",append]",
      vara[ii],varb[ii],dims="auto",intype="real",outtype="real",refim="auto",
      bwidth=0,btype="nearest",bpixval=0.,range+,verbose=no,exprdb="none")    
  }      
} else if (l_op=="*" || l_op=="/") {
  for(ii=1; ii<=numext; ii+=1) {
    imexpr("(a**2)*(c/a/a+d/b/b)"//l_op//"(b**2)",l_result//"["//l_var_ext//ii//",append]",
      scia[ii],scib[ii],vara[ii],varb[ii],dims="auto",intype="real",
      outtype="real",refim="auto",bwidth=0,btype="nearest",bpixval=0.,
      range+,verbose=no,exprdb="none")    
  }
}          

for(ii=1; ii<=numext; ii+=1) {
  addmasks(dqa[ii]//dqb[ii],l_result//"["//l_dq_ext//ii//",append]",
    dqexpr,flags=" ")
}

clean:
if(status==0)
   printlog("GEMARITH exit status: good",l_logfile,l_verbose)
else if (status>0)
   printlog("GEMARITH exit status: error",l_logfile,yes)

delete(tmp,ver-, >& "dev$null")
printlog("-------------------------------------------------------------\
-------------------",l_logfile,l_verbose)

end
