# Copyright(c) 2002-2003 Association of Universities for Research in Astronomy, Inc.

procedure gfskysub (inimages)

# Subtract a combined sky spectrum from a GMOS IFU datacube
# 
# Version   Sept 20, 2002 BM v1.4 release
#           Aug 25, 2003  KL IRAF2.12 - new parameter, addonly, in hedit

string inimages     {"",prompt="Input images"}
string outimages    {"",prompt="Output images"}
string outpref      {"s",prompt="Prefix for output images"}
string expr         {"XINST > 10.",prompt="Expression for selection of sky spectra"}
string combine      {"average",enum="average|median",prompt="Combine operation"}
string reject       {"avsigclip",enum="none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip",prompt="Rejection algorithm"}
string scale        {"none",prompt="Image scaling"}
string zero         {"none",prompt="Image zeropoint offset"}
string weight       {"none",prompt="Image weights"}
real   lthreshold   {INDEF,prompt="Lower threshold"}
real   hthreshold   {INDEF,prompt="Upper threshold"}
int    nlow         {1,min=0,prompt="minmax: Number of low pixels to reject"}
int    nhigh        {1,min=0,prompt="minmax: Number of high pixels to reject"}
int    nkeep        {0,min=0,prompt="Minimum to keep or maximum to reject"}  
bool   mclip        {yes,prompt="Use median in sigma clipping algorithms?"}
real   lsigma       {3.,prompt="Lower sigma clipping factor"}
real   hsigma       {3.,prompt="Upper sigma clipping factor"}
string key_ron      {"RDNOISE",prompt="Keyword for readout noise in e-"}
string key_gain     {"GAIN",prompt="Keyword for gain in electrons/ADU"}
string snoise       {"0.0",prompt="ccdclip: Sensitivity noise (electrons)"}
real   sigscale     {0.1,prompt="Tolerance for sigma clipping scaling correction"}
real   pclip        {-0.5,prompt="pclip: Percentile clipping parameter"}
real   grow         {0.0,prompt="Radius (pixels) for neighbor rejection"}
real   blank        {0.0,prompt="Value if there are no pixels"}
string  sci_ext     {"SCI",prompt="Name of science extension"}
string  var_ext     {"VAR",prompt="Name of variance extension"}
string  dq_ext      {"DQ",prompt="Name of data quality extension"}
string logfile      {"",prompt="Logfile name"}
bool   fl_inter     {yes,prompt="Select sky spectra interactively?"}
bool   verbose      {yes,prompt="Verbose?"}
int    status       {0,prompt="Exit status (0=good)"}
struct *scanfile1   {"",prompt="Internal use only"}
struct *scanfile2   {"",prompt="Internal use only"}

begin

# Local variables for input parameters
string l_inimages,l_outimages,l_prefix,l_logfile
string l_sci_ext, l_var_ext, l_dq_ext
string l_expr
bool l_verbose,l_fl_inter, l_mclip
string l_combine, l_reject, l_scale, l_zero, l_weight
string l_key_ron, l_key_gain, l_snoise, l_gain, l_rdnoise
real   l_lthreshold, l_hthreshold, l_lsigma, l_hsigma
real   l_grow, l_sigscale, l_pclip, l_blank
int    l_nlow, l_nhigh, l_nkeep
struct l_struct

# Other local variables
string inlist,outlist,temp1,temp2,temp3,mdf
string img,outimg,suf,aplist,apsel,range,sdum,sky
int nbad,nin,nout,nsky,prev,rstart,nrange,dum,nextnd
bool useprefix
struct sdate,sline

# Initialize exit status
status=0

# cache some parameter files
cache("imgets","gmos","hedit","gimverify","tinfo","tabpar")

# Initialize local variables
l_inimages=inimages ; l_outimages=outimages ; l_prefix=outpref
l_logfile=logfile ; l_verbose=verbose ; l_fl_inter=fl_inter
l_sci_ext=sci_ext ; l_var_ext=var_ext ; l_dq_ext=dq_ext 
l_expr=expr
l_combine=combine ; l_reject=reject
l_scale=scale ; l_zero=zero ; l_weight=weight
l_lthreshold=lthreshold ; l_hthreshold=hthreshold
l_nlow=nlow ; l_nhigh=nhigh ; l_nkeep=nkeep
l_lsigma=lsigma ; l_hsigma=hsigma ; l_mclip=mclip
l_gain=key_gain ;l_snoise=snoise ; l_rdnoise=key_ron
l_sigscale=sigscale ; l_pclip=pclip ; l_grow=grow ; l_blank=blank

# Start logfile
gemlogname(logpar=l_logfile,package="gmos")
if (gemlogname.status != 0) {
    goto error
}
l_logfile=gemlogname.logname

printlog("-------------------------------------------------------------------------------",l_logfile,l_verbose)
date | scan(sdate)
printlog("GFSKYSUB -- "//sdate,l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)

#The usual kind of checks to make sure everything we need is specified. 
nbad=0
if(l_inimages=="" || l_inimages==" ") {
  printlog("ERROR - GFSKYSUB: Input spectra is an empty string",l_logfile,yes)
  nbad=nbad+1 
}

if(substr(l_inimages,1,1)=="@") {
   inlist=substr(l_inimages,2,strlen(l_inimages))
   if(!access(inlist)) {
      printlog("ERROR - GFSKYSUB: Input list "//inlist//" not found",l_logfile,verbose+)
      nbad=nbad+1
   }
}

if((l_outimages=="" || l_outimages==" ") && (l_prefix=="" || l_prefix==" ")) {
    printlog("ERROR - GFSKYSUB: Neither the output spectra nor prefix is specified.",l_logfile,yes)
    nbad=nbad+1
} else if ((l_outimages!="" && l_outimages!=" ")) {
    useprefix=no
} else {
    useprefix=yes
}

if(substr(l_outimages,1,1)=="@") {
   outlist=substr(l_outimages,2,strlen(l_outimages))
   if(!access(outlist)) {
      printlog("ERROR - GFSKYSUB: Output list "//outlist//" not found",l_logfile,yes)
      nbad=nbad+1
   }
}

temp1=mktemp("tmpin")
temp2=mktemp("tmpout") 
temp3=mktemp("tmpfilelist") 
mdf=mktemp("tmpmdf")
apsel=mktemp("tmpapsel")
sky=mktemp("tmpsky")
files(l_inimages,sort-, > temp1)
count(temp1) | scan(nin)
if(!useprefix) { 
    files(l_outimages,sort-, > temp2)
    count(temp2) | scan(nout)
    if (nin != nout) {
        printlog("ERROR - GFSKYSUB: Different number of input and output spectra",l_logfile,yes)
        nbad=nbad+1
    }
} else {
   files(l_prefix//"//@"//temp1,sort-, > temp2)
}

# check existence of output files
scanfile2=temp2
while(fscan(scanfile2,img) != EOF) {
   if (imaccess(img)) {
       printlog("ERROR - GFSKYSUB: "//img//" already exists",l_logfile,yes)
       nbad=nbad+1
   }
}
scanfile2=""

#i=0
scanfile1=temp1
while(fscan(scanfile1,img) !=EOF) { 
    gimverify(img)
    if (gimverify.status>0) {
         nbad=nbad+1
    } else {
        #i=i+1
        # name w/o suffix
        #infile[i]=gimverify.outname
    }
#end of while loop over images. 
}
scanfile1=""

#If anything was wrong then exit. 
if (nbad > 0) { 
    printlog("ERROR - GFSKYSUB: "//nbad//" errors found with input parameters. Exiting.",l_logfile,yes) 
    goto error
} 

#If we are here then everything should be OK. 
#Write all the relevant info to the logfile:
#
printlog("inimages = "//l_inimages,l_logfile,l_verbose) 
printlog("outimages = "//l_outimages,l_logfile,l_verbose) 
printlog("outpref = "//l_prefix,l_logfile,l_verbose) 
printlog(" ",l_logfile,l_verbose)

# Make list file for main loop
join(temp1//","//temp2,delim=" ",missing="Missing",maxchar=161,shortest=yes,verbose=no, > temp3)
scanfile1=temp3 
while(fscan(scanfile1,img,outimg) !=EOF) { 
    # check .fits
    suf=substr(img,strlen(img)-4,strlen(img))
    if (suf!=".fits") {
      img=img//".fits"
    }

    suf=substr(outimg,strlen(outimg)-4,strlen(outimg))
    if (suf!=".fits") {
      outimg=outimg//".fits"
    }

    # copy image to output
    copy(img,outimg,verbose-)
 
    # Select the sky spectra
    tcalc(outimg//"[MDF,overwrite+]","BEAM","if BEAM != -1 && "//l_expr//" then 0 else BEAM")
    tselect(outimg//"[MDF]",mdf//".fits","BEAM >= 0")
    aplist=""
    tinfo(mdf//".fits",ttout-)
    nsky=0
    prev=0
    range=""
    nrange=0
    for (i=1; i<=tinfo.nrows; i+=1) {
        tabpar(mdf//".fits","BEAM",i)
        if (tabpar.value=="0") {
            if (prev==0) {
                prev=i
                rstart=i
                nrange=1
            } else if (i-prev != 1) {
                range=str(rstart)
                if (nrange>1) {
                    range=range//"-"//str(prev)
                }
                if (aplist=="") {
                    aplist=range
                } else {
                    aplist=aplist//","//range
                }
                if (i==tinfo.nrows) {
                    aplist=aplist//","//i
                }
                prev=i
                rstart=i
                range=""
                nrange=1
            } else if (i==tinfo.nrows) {
                range=str(rstart)
                if (nrange>1) {
                    range=range//"-"//str(i)
                } 
                if (aplist=="") {
                    aplist=range
                } else {
                    aplist=aplist//","//range
                }               
            } else {
                nrange=nrange+1
                prev=i
            }
            nsky=nsky+1
        }
    }
    delete(mdf//".fits",verify-)
    printlog("Found "//nsky//" sky spectra",l_logfile,l_verbose)
    #print (aplist)
    if (l_fl_inter) {
        specplot(outimg//"["//l_sci_ext//",1]",apertures=aplist,autolayout-,logfile=apsel)

        # Make new aperture list 
        aplist=""
        nsky=0
        prev=0
        range=""
        nrange=0
        scanfile2=apsel
        #read header lines
        for (i=1; i<=5; i+=1) {
            dum=fscan(scanfile2,sline)
        }
        dum=fscan(scanfile2,sline)
        while (sline != "") {
            print(sline) | translit("STDIN","()"," ",delete-) | scan(dum,sdum,i)
            if (prev==0) {
                prev=i
                rstart=i
                nrange=1
            } else if (i-prev != 1) {
                range=str(rstart)
                if (nrange>1) {
                    range=range//"-"//str(prev)
                }
                if (aplist=="") {
                    aplist=range
                } else {
                    aplist=aplist//","//range
                }
                prev=i
                rstart=i
                range=""
                nrange=1
            } else {
                nrange=nrange+1
                prev=i
            }
            nsky=nsky+1
            dum=fscan(scanfile2,sline)
        }
        range=str(rstart)
        if (nrange>1) {
            range=range//"-"//str(prev)
        }
        if (aplist=="") {
            aplist=range
        } else {
            aplist=aplist//","//range
        }
        scanfile2=""    
        #print(aplist)
    }

    # Make the combined sky
    scombine(outimg//"["//l_sci_ext//",1]",sky,noutput="",logfile="",
    apertures=aplist,group="images",comb=l_combine,reject=l_reject,
    scale=l_scale,zero=l_zero,weight=l_weight,lthreshold=l_lthreshold,
    hthreshold=l_hthreshold,nlow=l_nlow,nhigh=l_nhigh,nkeep=l_nkeep,
    mclip=l_mclip,lsigma=l_lsigma,hsigma=l_hsigma,rdnoise=l_rdnoise,
    gain=l_gain,snoise=l_snoise,pclip=l_pclip,grow=l_grow,blank=l_blank)

    # Subtract the sky
    imarith(outimg//"["//l_sci_ext//",1]","-",sky//".fits",outimg//"["//l_sci_ext//",1,overwrite+]")
    date | scan(l_struct)
    gemhedit(outimg//"[0]","GFSKYSUB",l_struct,"Time stamp for GFSKYSUB")
    gemhedit(outimg//"[0]","GEM-TLM",l_struct,"Last modification with GEMINI")
    
    # Put sky image in output
    imcopy(sky//".fits",outimg//"[SKY,1,append]",verbose-)
    imgets(outimg//"[0]","NEXTEND")
    nextnd=int(imgets.value)+1
    hedit(outimg//"[0]","NEXTEND",nextnd,add-,addonly-,del-,ver-,show-,update+)

    delete(apsel//","//sky//".fits",verify-, >>& "dev$null")
}
scanfile1=""

goto clean

error:
    status=1

clean:
    delete(temp1//","//temp2//","//temp3,verify-, >& "dev$null") 
    printlog("",l_logfile,l_verbose)
    scanfile1="" ; scanfile2=""
    if(status==0)
    printlog("GFSKYSUB exit status: good",l_logfile,l_verbose)
    else
    printlog("GFSKYSUB exit status: error",l_logfile,l_verbose)
    printlog("-------------------------------------------------------------------------------",l_logfile,l_verbose)

end
