# Copyright(c) 2003 Association of Universities for Research in Astronomy, Inc.

procedure miflat(inimage1,inimage2,outimage)

# This routine makes a flat field file from two images, which must be 
# the output of either "tprepare" or "mprepare".
#
# Version:  Sept 29, 2003  KV wrote the first version.
#           Oct   3, 2003  KV made small syntax changes in the parameters
#	    Oct 29, 2003  KL IRAF2.12 - new parameters
#			       imstat: nclip, lsigma, usigma, cache

char inimage1      {"",prompt="First input T-ReCS or Michelle flat-field image"}
char inimage2      {"",prompt="Second input T-ReCS or Michelle flat-field image"}
char  rawpath      {"",prompt="Path for input raw images"}
char  outimage     {"",prompt="Output image name"}
char  logfile      {"",prompt="Logfile name"}
bool  verbose      {yes,prompt="Verbose logging yes/no?"}
int status         {"",prompt="Exit error status: (0=good, >0=bad)"}
struct* scanfile   {"",prompt="Internal use only"}

begin

char l_inputimage1,l_inputimage2,l_outputimage
char l_rawpath
char l_logfile
bool l_verbose

char tmpimage,header1,header2,tmpstring
int naxis[4],iaxis,inst1,inst2,i,j
real mean1,mean2

struct l_struct

l_inputimage1=inimage1
l_inputimage2=inimage2
l_outputimage=outimage
l_rawpath=rawpath
l_logfile=logfile
l_verbose=verbose

status=0

# Check the rawpath name for a final /
if(substr(l_rawpath,(strlen(l_rawpath)),(strlen(l_rawpath))) != "/") {
   l_rawpath=l_rawpath//"/"
}
if(l_rawpath=="/" || l_rawpath==" ")
  l_rawpath=""

# Set up the logfile
#
# First, check for package logfile if the logfile parameter is blank; if it 
# is empty use midir.logfile as the default.
#
if((l_logfile=="") || (l_logfile==" ")) {

  l_logfile=midir.logfile

  if((l_logfile=="") || (l_logfile==" ")) {
    l_logfile="midir.log"
    printlog("Warning - MIFLAT:  Both miflat.logfile and midir.logfile are undefined.  Using default file midir.log.",logfile=l_logfile, verbose+)
  }
}

# Open log file
date | scan(l_struct)
printlog("----------------------------------------------------------------------------",logfile=l_logfile, verbose=l_verbose)
printlog("Starting MIFLAT -- "//l_struct, logfile=l_logfile, verbose=l_verbose)
printlog("",logfile=l_logfile,verbose=l_verbose)
printlog("First input T-ReCS or Michele image  = "//l_inputimage1,l_logfile,l_verbose)
printlog("Second input T-ReCS or Michele image = "//l_inputimage2,l_logfile,l_verbose)
printlog("Output image                         = "//l_outputimage,l_logfile,l_verbose)
printlog("Path for input raw images            = "//l_rawpath,l_logfile,l_verbose)
printlog("Logfile name                         = "//l_logfile,l_logfile,l_verbose)
printlog("Verbose logging?                     = "//l_verbose,l_logfile,l_verbose)
printlog(" ",logfile=l_logfile, verbose=l_verbose)

if (!imaccess(l_rawpath//l_inputimage1//"[0]")) {
  printlog("ERROR - MIFLAT: Input image "//l_rawpath//l_inputimage1//" was not found.",logfile=l_logfile,verbose+)
  status=1
  goto exit
}
if (!imaccess(l_rawpath//l_inputimage2//"[0]")) {
  printlog("ERROR - MIFLAT: Input image "//l_rawpath//l_inputimage2//" was not found.",logfile=l_logfile,verbose+)
  status=1
  goto exit
}
if (imaccess(l_outputimage)) {
  printlog("ERROR - MIFLAT: Output image "//l_outputimage//" already exists.",logfile=l_logfile,verbose+)
  status=1
  goto exit
}


header1=l_rawpath//l_inputimage1//"[0]"
header2=l_rawpath//l_inputimage2//"[0]"

imgets(header1,"INSTRUME",>& "dev$null")

inst1=0
if (imgets.value == "TReCS") {
  inst1=1
}
if (imgets.value == "Michelle") {
  inst1=2
}

imgets(header2,"INSTRUME",>& "dev$null")
inst2=0
if (imgets.value == "TReCS") {
  inst2=1
}
if (imgets.value == "Michelle") {
  inst2=2
}

if (inst1 == 0) {
  printlog("ERROR - MIFLAT: Input image "//l_inputimage1//" does not seem to be a T-ReCS or Michelle file.",logfile=l_logfile,verbose+)
  status=1
  goto exit
}

if (inst2 == 0) {
  printlog("ERROR - MIFLAT: Input image "//l_inputimage2//" does not seem to be a T-ReCS or Michelle file.",logfile=l_logfile,verbose+)
  status=1
  goto exit
}

if (inst1 != inst2) {
  printlog("ERROR - MIFLAT: Input images "//l_inputimage1//" and "//l_inputimage2//" do not come from the same instrument!.",logfile=l_logfile,verbose+)
  status=1
  goto exit
}

if (inst1 == 1) {
  imgets(header1,"TPREPARE",>& "dev$null")
  if (imgets.value == "0") {
    printlog("ERROR - MIFLAT: Input image "//l_inputimage1//" has not been processed with TPREPARE.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
  imgets(header2,"TPREPARE",>& "dev$null")
  if (imgets.value == "0") {
    printlog("ERROR - MIFLAT: Input image "//l_inputimage2//" has not been processed with TPREPARE.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
}

if (inst1 == 2) {
  imgets(header1,"MPREPARE",>& "dev$null")
  if (imgets.value == "0") {
    printlog("ERROR - MIFLAT: Input image "//l_inputimage1//" has not been processed with MPREPARE.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
  imgets(header2,"MPREPARE",>& "dev$null")
  if (imgets.value == "0") {
    printlog("ERROR - MIFLAT: Input image "//l_inputimage2//" has not been processed with MPREPARE.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
}

header1=l_rawpath//l_inputimage1//"[1]"
if (imaccess(header1)) {
  imgets(header1,"i_naxis")
  iaxis=int(imgets.value)
  if (iaxis < 2 || iaxis > 4) {
    printlog("ERROR - MIFLAT: Input image "//l_inputimage1//" has a bad number of dimensions ("//str(iaxis)//") for extension 1.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
  naxis[1]=0
  naxis[2]=0
  naxis[3]=0
  naxis[4]=0

  for (j=1; j <= iaxis; j+=1) {
    tmpstring="naxis"//str(j)
    imgets(header1,tmpstring)
    naxis[j]=int(imgets.value)
    if (naxis[j] < 0) {
      printlog("ERROR - MIFLAT: Image "//l_inputimage1//" has negative image dimension(s) in extension 1.",logfile=l_logfile,verbose+)
      status=1
      goto exit
    }
  }
  if (iaxis > 2 || naxis[1] != 320 || naxis[2] != 240) {
    printlog("ERROR - MIFLAT: Image "//l_inputimage1//" has bad image dimension(s) in extension 1.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
}
else {
  printlog("ERROR - MIFLAT: A problem occured when reading extension 1 of input image "//l_inputimage1//".",logfile=l_logfile,verbose+)
  status=1
  goto exit
}

imstat(header1,field="midpt",lower=INDEF,upper=INDEF,nclip=0,lsigma=INDEF,
    usigma=INDEF,binwidth=0.1,format-,cache-) | scanf("%f",mean1)

header2=l_rawpath//l_inputimage2//"[1]"
if (imaccess(header2)) {
  imgets(header2,"i_naxis")
  iaxis=int(imgets.value)
  if (iaxis < 2 || iaxis > 4) {
    printlog("ERROR - MIFLAT: Input image "//l_inputimage2//" has a bad number of dimensions ("//str(iaxis)//") for extension 1.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
  naxis[1]=0
  naxis[2]=0
  naxis[3]=0
  naxis[4]=0

  for (j=1; j <= iaxis; j+=1) {
    tmpstring="naxis"//str(j)
    imgets(header2,tmpstring)
    naxis[j]=int(imgets.value)
    if (naxis[j] < 0) {
      printlog("ERROR - MIFLAT: Image "//l_inputimage2//" has negative image dimension(s) in extension 1.",logfile=l_logfile,verbose+)
      status=1
      goto exit
    }
  }
  if (iaxis > 2 || naxis[1] != 320 || naxis[2] != 240) {
    printlog("ERROR - MIFLAT: Image "//l_inputimage2//" has bad image dimension(s) in extension 1.",logfile=l_logfile,verbose+)
    status=1
    goto exit
  }
}
else {
  printlog("ERROR - MIFLAT: A problem occured when reading extension 1 of input image "//l_inputimage2//".",logfile=l_logfile,verbose+)
  status=1
  goto exit
}

imstat(header2,field="midpt",lower=INDEF,upper=INDEF,nclip=0,lsigma=INDEF,
    usigma=INDEF,binwidth=0.1,format-,cache-) | scanf("%f",mean2)
printf("mean value 1 = %f\n",mean2)

tmpimage=mktemp("tmpimage")

if (mean1 > mean2) {
  imarith(header1//"[*,*]","-",header2//"[*,*]",tmpimage,verbose-)
}
else {
  imarith(header2//"[*,*]","-",header1//"[*,*]",tmpimage,verbose-)
}

imstat(tmpimage,field="midpt",lower=INDEF,upper=INDEF,nclip=0,lsigma=INDEF,
    usigma=INDEF,binwidth=0.1,format-,cache-) | scanf("%f",mean1)

if (mean1 == 0.) {
  printlog("ERROR - MIFLAT: A problem occured when making the flat field image (median value is zero).",logfile=l_logfile,verbose+)
  status=1
  goto exit
}

imarith(tmpimage,"/",mean1,l_outputimage,verbose-)

exit:

  delete("tmpimage*.fits",ver-,>>& "/dev/null")

  date | scan(l_struct)
  if (status == 0) {
    printlog("MIFLAT: exit status => good.  "//l_struct,logfile=l_logfile, verbose=l_verbose)
  }
  else {
    printlog("MIFLAT: exit status => error.  "//l_struct,logfile=l_logfile, verbose=l_verbose)
  }
  printlog("----------------------------------------------------------------------------", logfile=l_logfile, verbose=l_verbose)
  printlog(" ",logfile=l_logfile, verbose=l_verbose)

end
