# Copyright(c) 2001-2003 Association of Universities for Research in Astronomy, Inc.

procedure mprepare(inimages)

# Procedure to take raw Michelle data, with n MEF extensions (where n is the
# number of nodsets), and reformat the data so that it is consistent with
# the higher level data reduction image format.

# Data format is flipped from difference frame (ext 1), nod A (ext 2), nod
# B (ext 3) to nod A (ext 1), nod B (ext2), difference frame (ext 3).

# Version Sep  3, 2003  TB, first write
#         Oct 29, 2003  KL  IRAF 2.12 - new parameters
#                             hedit: addonly

char  inimages     {prompt="Input Michelle image(s)"}               # OLDP-1
char  rawpath      {"",prompt="Path for input raw images"}      # OLDP-1
char  outimages    {"",prompt="Output image(s)"}                # OLDP-1
char  outprefix    {"m",prompt="Prefix for output image(s)"}    # OLDP-1
char  logfile      {"",prompt="Logfile"}                        # OLDP-1
char  sci_ext      {"SCI",prompt="Name or number of science extension"}     # OLDP-3
char  key_sat      {"SATURATI",prompt="New header keyword for saturation (ADU)"}  # OLDP-3
char  key_filter   {"FILTER",prompt="New header keyword for parsed filter"} # OLDP-3
#char  specsec      {"[1:1024,1:1024]",prompt="Default region of image to use for spectra when database cannot be found"}  # OLDP-3
char  database     {"midir$midir.dat",prompt="MPREPARE database to use"}  # OLDP-3
bool  verbose      {yes,prompt="Verbose"}            # OLDP-2
int   status       {0,prompt="Exit status (0=good)"} # OLDP-4
struct *scanfile   {"",prompt="Internal use only"}   # OLDP-4

begin
char l_inimages, l_outimages, l_filter, l_filter1, l_filter2, l_filter3
char l_rawpath, l_prefix, l_logfile, l_temp, tmpin, tmpfile1, tmpfile2, tmpfile, tmphead
char in[1000], inpath[1000], out[1000]
char tmpsci, tmpvar, tmpdq, tmpinimage, header
int  i, nimages, noutimages, maxfiles, nbad, n_j, nout, ninp, modeflag
char  l_key_filter, l_sci_ext
char l_key_sat, l_pupil
char  l_tempexpression, l_database, l_fpmask, l_camera
int  l_sat, l_linlimit, next_inp
real l_exptime
bool l_verbose,  alreadyfixed[1000], bad
bool oldtimer
int  bpmxsize, bpmysize, xsize, ysize, sx1, sx2, sy1, sy2, next_sci
int  next_anod, next_bnod, j
struct l_struct

status=0
maxfiles=1000
# cache imgets - used throughout the script
cache("imgets")

# set the local variables
l_inimages=inimages ; l_outimages=outimages ; l_rawpath=rawpath
l_verbose=verbose ; l_prefix=outprefix ; l_logfile=logfile
l_key_filter=key_filter
l_key_sat=key_sat ; 
l_sci_ext=sci_ext 
l_database=database 

# open temp files
tmpfile = mktemp("tmp1")
tmpfile1 = mktemp("tmp2")
tmpfile2 = mktemp("tmp2")
tmpin = mktemp("tmpin")
tmpsci = mktemp("tmpsci")
tmpvar = mktemp("tmpvar")
tmpdq = mktemp("tmpdq")
tmpinimage = mktemp("tmpinimage")
tmphead = mktemp("tmphead")

# Test that name of logfile makes sense
#cache("midir")
print(l_logfile) | scan(l_logfile)
if(l_logfile=="" || l_logfile==" ") {
   l_logfile=midir.logfile
   print(l_logfile) | scan(l_logfile)
   if((l_logfile=="") || (l_logfile==" ")) {
      l_logfile="midir.log"
      printlog("WARNING - MPREPARE: Both mprepare.logfile and midir.logfile are empty.",logfile=l_logfile, verbose+)
      printlog("                  Using default file midir.log.",
         logfile=l_logfile, verbose+)
   }
}
# Open log file
date | scan(l_struct)
printlog("----------------------------------------------------------------------------",l_logfile,l_verbose)
printlog("MPREPARE -- "//l_struct,l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)

#----------------------------------------------------------------------
# Check the rawpath name for a final /
if(substr(l_rawpath,(strlen(l_rawpath)),(strlen(l_rawpath))) != "/") {
   l_rawpath=l_rawpath//"/"
}
if(l_rawpath=="/" || l_rawpath==" ")
  l_rawpath=""

#-----------------------------------------------------------------------
# Load up arrays of input name lists
# This version handles both *s, / and commas in l_inimages

ninp = 0
nout=0

# check that list file exists
if(substr(l_inimages,1,1)=="@") {
  l_temp=substr(l_inimages,2,strlen(l_inimages))
  if(!access(l_temp) && !access(l_rawpath//l_temp)) {
    printlog("ERROR - MPREPARE:  Input file "//l_temp//" not found.",l_logfile,verbose+)
    status=1
    goto clean
  }
}


# parse wildcard and comma-separated lists
if(substr(l_inimages,1,1)=="@") {
  scanfile=substr(l_inimages,2,strlen(l_inimages))
} else {
  print(l_inimages, >tmpfile)
  sed("-e",'s/\,/\ /g',tmpfile) | words("STDIN", >tmpfile2)
  scanfile=tmpfile2
  delete(tmpfile,verify-, >>& "dev$null")
}  
while(fscan(scanfile,l_temp) != EOF) {
  files(l_rawpath//l_temp,sort-) | unique("STDIN", >> tmpfile)
}

delete(tmpfile2,verify-, >>& "dev$null")
scanfile=tmpfile

nimages=0
nbad=0
while(fscan(scanfile,l_temp) != EOF) {
  bad=no
# remove rawpath
  l_temp=substr(l_temp,strlen(l_rawpath)+1,strlen(l_temp))

# remove .fits if present
  if(substr(l_temp,strlen(l_temp)-4,strlen(l_temp)) == ".fits")
     l_temp=substr(l_temp,1,(strlen(l_temp)-5))
# check to see if the file exists (or if it's not MEF FITS)
  gimverify(l_rawpath//l_temp)
  if(gimverify.status==1) {
     printlog("ERROR - MPREPARE: Input image "//l_rawpath//l_temp//" not found.",l_logfile,verbose+)
     nbad+=1
     bad=yes
  } else if(gimverify.status>1) {
     printlog("ERROR - MPREPARE: Input image not multiple-extension FITS.",l_logfile,verbose+)
     nbad+=1
     bad=yes
  }
  else {
    nimages=nimages+1
    if(nimages > maxfiles) {
       printlog("ERROR - MPREPARE: Maximum number of input images exceeded ("//str(maxfiles)//")",l_logfile,verbose+)
       status=1
       goto clean
    }
    # check to see if already fixed with MPREPARE
    if(!bad) {
       imgets(l_rawpath//l_temp//"[0]","MPREPARE", >>& "dev$null")
       if(imgets.value != "0") {
          alreadyfixed[nimages]=yes
          printlog("WARNING - MPREPARE: Image "//l_rawpath//l_temp//" already fixed using MPREPARE.",l_logfile,verbose+) 
          printlog("                    Data scaling, correction for number of reads, and header",l_logfile,verbose+)
          printlog("                    updating not performed.",l_logfile,verbose+)
       } else
       alreadyfixed[nimages]=no
       in[nimages]=l_temp
    } # end if(!bad)
    # trim off path from in[i], if present
    i=strlen(in[nimages])-1
    while(i>1) {
      if(substr(in[nimages],i,i) == "/") {
        inpath[nimages]=substr(in[nimages],1,i)
        in[nimages]=substr(in[nimages],i+1,strlen(in[nimages]))
        i=0
      }
      i=i-1
    } # end while(i>1) path-chopping loop
  } # end else
} # end while(fscan) loop

# Exit if problems found
if (nbad > 0) {
  printlog("ERROR - MPREPARE: "//nbad//" image(s) either do not exist or are not MEF files.",l_logfile,verbose+)
  status=1
  goto clean
}

printlog("Processing "//nimages//" files",l_logfile,l_verbose)
scanfile="" ; delete(tmpfile//","//tmpin,ver-, >>& "dev$null")

# Now for the output images
# outimages could contain legal * if it is of a form like %st%stX%*.imh


noutimages=0
nbad=0
print(l_outimages) | scan(l_outimages)
#print(l_outimages)

if(l_outimages!="" && l_outimages!=" ") {
  if(substr(l_outimages,1,1)=="@") {
    scanfile=substr(l_outimages,2,strlen(l_outimages))
  } else { 
    files(l_outimages,sort-) | unique("STDIN", > tmpfile)
    scanfile=tmpfile
  }

  while(fscan(scanfile,l_temp) != EOF) {
# remove .fits if present
    if(substr(l_temp,strlen(l_temp)-4,strlen(l_temp)) == ".fits")
       l_temp=substr(l_temp,1,(strlen(l_temp)-5))
    noutimages=noutimages+1
    if(noutimages > maxfiles) {
       printlog("ERROR - MPREPARE: Maximum number of output images exceeded ("//str(maxfiles)//")",l_logfile,verbose+ )
       status=1
       goto clean
    }
    out[noutimages]=l_temp//".fits" 
    if(imaccess(out[noutimages])) {
      printlog("ERROR - MPREPARE: Output image "//out[noutimages]//" exists",
         l_logfile,verbose+)
      nbad+=1
    }
  } # end while
}

scanfile="" ; delete(tmpfile,ver-, >>& "dev$null")

# Exit if problems found
if (nbad > 0) {
  printlog("ERROR - MPREPARE: "//nbad//" image(s) already exist.",l_logfile,verbose+)
  status=1
  goto clean
}

# if there are too many or too few output images, and any defined
# at all at this stage - exit with error
if(nimages!=noutimages && l_outimages!="") {
  printlog("ERROR - MPREPARE: Number of input and output images unequal.",
     l_logfile,verbose+)
  status=1
  goto clean
}

# If prefix is to be used instead
if(l_outimages=="" || l_outimages==" ") {
  print(l_prefix) | scan(l_prefix)
  if(l_prefix=="" || l_prefix==" ") {
     printlog("ERROR - MPREPARE: Neither output image name nor output prefix is defined.",l_logfile,verbose+)
     status=1
     goto clean
  }
  i=1
  nbad=0
  while(i<=nimages) {
    out[i]=l_prefix//in[i]//".fits"
    if(imaccess(out[i])) {
       printlog("ERROR - MPREPARE: Output image "//out[i]//" already exists.",
          l_logfile,verbose+)
       nbad+=1
    }
    i=i+1
  }
  if (nbad > 0) {
    printlog("ERROR - MPREPARE: "//nbad//" image(s) already exist.",l_logfile,verbose+)
    status=1
    goto clean
  }
}


i=1

while(i<=nimages) { 
#in[i]=inpath[i]+in[i]+".fits"

if(l_rawpath=="" || l_rawpath==" "){
    in[i]=in[i]+".fits"
} else {
    in[i]=l_rawpath//in[i]+".fits"
  }


# Determine the number of extensions/nod sets.

  header=in[i]//"[0]"

      imgets(header,"MODE", >& "dev$null")
      if (imgets.value == "0") {
        printlog("ERROR - MPREPARE: could not find the MODE from the primary header.",l_logfile,l_verbose)
        status=status+1
     goto nextimage
      }

      modeflag=0
      if (imgets.value == "chop-nod") modeflag=1
      if (imgets.value == "ndchop") modeflag=1
      if (imgets.value == "chop") modeflag=2
      if (imgets.value == "nod") modeflag=3
      if (imgets.value == "ndstare") modeflag=4
      if (imgets.value == "stare") modeflag=4
      if (modeflag == 0) {
        printlog("ERROR - MPREPARE: Unrecognised MODE ("//imgets.value//") in the primary header.",l_logfile,l_verbose)
        status=status+1
        goto nextimage
      }

  imgets(header,"NUMEXT", >>& "dev$null")
  next_inp=int(imgets.value)


# Update EXTNAME and EXTVER, assuming all extensions but the MDF are SCI
    n_j=1
    next_sci = 0
     if ((modeflag == 4) && (next_inp != 1)) {
        printlog("ERROR - MPREPARE: Number of extensions in the primary header not consistent with the MODE (stare).",l_logfile,l_verbose)
        status=status+1
        goto nextimage
      }
     if ((modeflag == 2) && (next_inp != 1)) {
        printlog("ERROR - MPREPARE: Number of extensions in the primary header not consistent with the MODE (chop).",l_logfile,l_verbose)
        status=status+1
        goto nextimage
      }


 
    while (n_j<=next_inp) {

#            hedit(out[i]//"["//n_j//"]","EXTNAME",l_sci_ext,add+,addonly-,
#		delete-,verify-,show-,update+)
#            hedit(out[i]//"["//n_j//"]","EXTVER",n_j,add+,addonly-,delete-,
#		verify-,show-,update+)        
            next_sci = next_sci + 1     
        n_j = n_j+1
    }
    flpr # necessary to work around an IRAF bug 
    

    if (next_sci > next_inp) {
       printlog("ERROR - MPREPARE: Maximum number of "//l_sci_ext//" extensions exceeded.",l_logfile,verbose+)
       imdelete (out[i],ver-, >>& "dev$null")
       goto crash
    }
     

#--------------------------------------------------------------------------
# start output
printlog(" ",l_logfile,l_verbose)
printlog("  m      input file -->      output file",l_logfile,l_verbose)
#printlog("                 filter     focal plane       input BPM   RON  gain     sat",l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)

#Start on the main mprepare steps - rearrange the order of images within the extensions.
#and convert the 4D format to 3D format.

#if(l_outimages=="" || l_outimages==" ") {
#    out[i]=l_prefix+in[i]+".fits"
#}
    n_j=1
    next_sci = 0
#Copy PHU into new file
    fxcopy(in[i]//"[0]",out[i])

   while (n_j<=next_inp) {

    if (modeflag == 4) {
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
printlog("MPREPARE -- Prepared image for stare "//i,l_logfile,l_verbose)
delete(tmpfile//".fits",ver-)
delete(tmpfile//","//tmpfile1//","//tmphead,ver-, >>& "dev$null")
n_j=n_j+1
}
    if (modeflag == 3) {
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
printlog("MPREPARE -- Prepared image for nod "//n_j,l_logfile,l_verbose)
delete(tmpfile1//".fits",ver-)
n_j=n_j+1
}

    if ((modeflag == 2 || modeflag ==1) && next_inp == 1) {
#Re-arrange A nod frame
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,2,1].fits",tmpfile1//"[*,*,1]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,3,1].fits",tmpfile1//"[*,*,2]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,1,1].fits",tmpfile1//"[*,*,3]",ver-,)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
n_j=n_j+1
printlog("MPREPARE -- Rearranged image for chop frame "//i,l_logfile,l_verbose)
delete(tmpfile1//".fits",ver-)
}
    if (modeflag == 1 && next_inp != 1) {
#Re-arrange A nod frame
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,2,1].fits",tmpfile1//"[*,*,1]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,3,1].fits",tmpfile1//"[*,*,2]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,1,1].fits",tmpfile1//"[*,*,3]",ver-,)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
printlog("MPREPARE -- Rearranged image for nod "//n_j,l_logfile,l_verbose)
n_j=n_j+1
next_sci=next_sci+1
delete(tmpfile1//".fits",ver-)
#Re-arrange and invert beam for B nod frame
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,2,1].fits",tmpfile1//"[*,*,2]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,3,1].fits",tmpfile1//"[*,*,1]",ver-)      
    imarith(in[i]//"["//n_j//"]"//"[*,*,1,1].fits","*","-1",tmpfile2,ver-)
    imcopy(tmpfile2,tmpfile1//"[*,*,3]",ver-)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
printlog("MPREPARE -- Rearranged image for nod "//n_j,l_logfile,l_verbose)
n_j=n_j+1
next_sci=next_sci+1
delete(tmpfile1//".fits",ver-)
delete(tmpfile2//".fits",ver-)
#Take into account the fact that there might be a non-divisible by 4 number of nod pairs
    if (n_j < next_inp) {
#Re-arrange and invert beam for B nod frame
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,2,1].fits",tmpfile1//"[*,*,2]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,3,1].fits",tmpfile1//"[*,*,1]",ver-)      
    imarith(in[i]//"["//n_j//"]"//"[*,*,1,1].fits","*","-1",tmpfile2,ver-)
    imcopy(tmpfile2,tmpfile1//"[*,*,3]",ver-)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
printlog("MPREPARE -- Rearranged image for nod "//n_j,l_logfile,l_verbose)
#Re-arrange A nod frame
n_j=n_j+1
next_sci=next_sci+1
delete(tmpfile1//".fits",ver-)
delete(tmpfile2//".fits",ver-)
    imcopy(in[i]//"["//n_j//"]"//"[*,*,*,1].fits",tmpfile1,ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,2,1].fits",tmpfile1//"[*,*,1]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,3,1].fits",tmpfile1//"[*,*,2]",ver-)      
    imcopy(in[i]//"["//n_j//"]"//"[*,*,1,1].fits",tmpfile1//"[*,*,3]",ver-,)      
    fxinsert(tmpfile1//".fits",out[i]//"["//next_sci//"]",groups="",ver-)
printlog("MPREPARE -- Rearranged image for nod "//n_j,l_logfile,l_verbose)
n_j=n_j+1
next_sci=next_sci+1
delete(tmpfile1//".fits",ver-)
}}
}


#-------------
# Pack up the results and clean up

#  hedit(out[i]//"["//l_sci_ext//"]","EXTVER",1,add+,addonly-,delete-,verify-,
#	show-,update+)

#imdel(tmpinimage,verify-)

#-----------------
# Fix up the headers
#hedit(out[i]//"[0]","WMEF","",add-,addonly-,delete+,verify-,show-,update+)
#hedit(out[i]//"[0]","GEM-TLM","",add-,addonly-,delete+,verify-,show-,update+)
 
printlog("MPREPARE -- finished with image number: "//i,l_logfile,l_verbose)

printlog("",l_logfile,l_verbose)

date | scan(l_struct)

printlog("MPREPARE -- "//l_struct,l_logfile,l_verbose)

printlog("----------------------------------------------------------------------------",l_logfile,l_verbose)

# Time stamps
  date | scan(l_struct)
  printf("%-8s= \'%-18s\' / %-s\n","GEM-TLM",l_struct,"Last modification with GEMINI IRAF", >> tmphead)
  printf("%-8s= \'%-18s\' / %-s\n","MPREPARE",l_struct,"Time stamp for MPREPARE", >> tmphead)

   mkheader(out[i]//"[0]",tmphead,append+,verbose-)
   delete(tmphead,verify-, >>& "dev$null")

n_j=1
    while (n_j<=next_inp) {
 hedit(out[i]//"["//n_j//"]","AXISLAB4","",add-,addonly-,del+,ver-,show-,upd+)
n_j=n_j+1

#   printf("%3.0d %15s --> %16s \n",i,in[i],out[i]) | scan(l_struct)
#   printlog(l_struct,l_logfile,l_verbose)
#printlog("----------------------------------------------------------------------------",l_logfile,l_verbose)
    }

  nextimage:
    i=i+1


} # end loop

clean:
{
  if(status==0) {
    printlog(" ",l_logfile,l_verbose)
    printlog("MPREPARE exit status:  good.",l_logfile,l_verbose)
  }
  printlog("----------------------------------------------------------------------------",l_logfile,l_verbose)

scanfile="" 

delete(tmpin,ver-, >>& "dev$null")
delete(tmpfile//","//tmpfile1//","//tmphead,ver-, >>& "dev$null")
delete(tmpfile//","//tmpfile2//","//tmphead,ver-, >>& "dev$null")
imdelete(tmpsci//","//tmpvar//","//tmpdq,ver-, >>& "dev$null")

}

end
