# Copyright(c) 2003 Association of Universities for Research in Astronomy, Inc.

procedure tcheckstructure (image)

#
# This is a task that tests MEF file integrity for 
# TReCS files.  It checks the number of extensions and 
# the size of the extensions against the header keywords.
#
# It should work for the output files from "tprepare" as well 
# as the raw T-ReCS files.
#
# Version:   Oct 5,  2003  KV routine changed to not write to the terminal
#                            unless an error oocurs
#            Sept 3, 2003  KV original routine created

char image	{"",prompt="Image to check"}
int modeflag    {"",prompt="Exit observation mode flag"}
char logfile 	{"",prompt="Logfile name"}
bool verbose    {"no",prompt="Verbose?"}
int status      {"",prompt="Exit error status: (0=good, >0=bad)"}

begin

char l_image,mainlogfile,l_logfile,extname,tmpstring
struct l_struct
bool l_verbose
int nnods,nnodsets,nextends,savesets
int iaxis,naxis[4]
int i,j,jpre
l_image=image
l_logfile=logfile
l_verbose=verbose

mainlogfile=midir.logfile

# if logfile not defined, default to scriptname.log

if((l_logfile=="") || (l_logfile==" ")) {
  if (mainlogfile=="" || mainlogfile==" ") {
    l_logfile="midir.log"
  }
  else {
    l_logfile=mainlogfile
  }
}

# Open task log file
date | scan(l_struct)
printlog("----------------------------------------------------------------------------",logfile=l_logfile, verbose-)
printlog("Starting TCHECKSTRUCTURE -- "//l_struct, l_logfile, verbose-)

#check if image exists

if(!imaccess(l_image)) {
  printlog("ERROR - TCHECKSTRUCTURE: Image "//l_image//" is not found.",l_logfile,verbose+)
  status=1
  bye
}


# Now, read the main header for relevant values

modeflag=0
extname=l_image//"[0]"
if (!imaccess(extname)) {
  printlog("ERROR - TCHECKSTRUCTURE: Primary head unit for file "//l_image//" is not found",l_logfile,verbose+)
  status=1
  bye
}

imgets(extname,"INSTRUME",>& "dev$null")
if (imgets.value != "TReCS") {
  printlog("ERROR - TCHECKSTRUCTURE: Image "//l_image//" has INSTRUME keyword"//imgets.value//" rather than 'TReCS'.",l_logfile,verbose+)
  status=1
  bye
}

imgets(extname,"MISTACK",>& "dev$null")
if (imgets.value != "0") {
  printlog("This is a file from MISTACK.",l_logfile,l_verbose)
  status=1
  bye
}
imgets(extname,"MIREGISTER",>& "dev$null")
if (imgets.value != "0") {
  printlog("This is a file from MIREGISTER.",l_logfile,l_verbose)
  status=1
  bye
}

imgets(extname,"OBSMODE",>& "dev$null")
if (imgets.value == "chop-nod") {
  modeflag=1
}
if (imgets.value == "chop") {
  modeflag=2
}
if (imgets.value == "nod") {
  modeflag=3
}
if (imgets.value == "stare") {
  modeflag=4
}
if (modeflag == 0) {
  printlog("ERROR - TCHECKSTRUCTURE: Image "//l_image//" has unrecognised OBSMODE keyword.",l_logfile,verbose+)
  status=1
  bye
}

imgets(extname,"TPREPARE",>& "dev$null")
if (imgets.value == "0") {
  jpre=1
}
else {
  jpre=2
}

imgets(extname,"NNODS",>& "dev$null")
nnods=int(imgets.value)
imgets(extname,"NNODSETS",>& "dev$null")
nnodsets=int(imgets.value)
if (jpre == 1) {
  nextends=nnodsets*nnods
}
else {
  nextends=1
}
imgets(extname,"SAVESETS",>& "dev$null")
savesets=int(imgets.value)
if (nnods < 1 || nnods > 2 || nnodsets < 1 || savesets < 1) {
  tmpstring="Primary header error: NNODS = "//str(nnods)//" NNODSETS = "//str(nnodsets)//" SAVESETS = "//str(savesets)
  printlog(tmpstring,l_logfile,l_verbose)
  status=1
  bye
}

status=0

# Check the extensions and the dimension values

j= -10

for (i=1; i < 10000 ; i+=1) {
  extname=l_image//"["//i//"]"
  if (!imaccess(extname)) {
    j=i-1
    i=10001
  }
}

if (j != nextends) {
  tmpstring="The file has "//str(j)//" extensions.\nThe expected number of extensions is "//str(nextends)//"."
  printlog(tmpstring,l_logfile,l_verbose)
  status=1
  bye
}

for (i=1; i <= nextends; i+=1) {
  extname=l_image//"["//i//"]"
  if (imaccess(extname)) {
    imgets(extname,"i_naxis")
    iaxis=int(imgets.value)
    if (iaxis < 2 || iaxis > 4) {
      tmpstring="Image extension "//str(i)//" has NAXIS = "//str(iaxis)//".\nThe expected number is 2, 3 or 4"
      printlog(tmpstring,l_logfile,l_verbose)
      status=status+1
    }
    else {
      naxis[1]=0
      naxis[2]=0
      naxis[3]=0
      naxis[4]=0

      if (iaxis == 2 || iaxis == 3 && modeflag < 10) modeflag=modeflag+10
#
# add 10 to the modeflag if the file looks like an output file from 
# "tprepare".
#

      for (j=1; j <= iaxis; j+=1) {
        tmpstring="naxis"//str(j)
        imgets(extname,tmpstring)
        naxis[j]=int(imgets.value)
      }
# The following assumes that NOD mode observations have structure [320,240,1,nsavesets]: no chopping
# so only one chop position gives third dimension 1.
#
# This is to be verified.
#
      if (naxis[1] != 320 || naxis[2] != 240) {
        if (iaxis == 2) {
          tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//".\nThe expected numbers are [320,240]"
        }
        if (iaxis == 3) {
          tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//" "//str(naxis[3])//".\nThe expected numbers are [320,240,3]"
        }
        if (iaxis == 4) {
          if (modeflag == 1 || modeflag == 2) {
            tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//" "//str(naxis[3])//" "//str(naxis[4])//".\nThe expected numbers are [320,240,2,"//str(savesets)//"]"
          }
          if (modeflag == 3 || modeflag == 4) {
            tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//" "//str(naxis[3])//" "//str(naxis[4])//".\nThe expected numbers are [320,240,1,"//str(savesets)//"]"
          }
        }
        status=status+1
        printlog(tmpstring,l_logfile,l_verbose)
      }
      if (iaxis == 2 && modeflag != 4) {
        tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//".\nThe OBSMODE is not 'stare'."
        status=status+1
        printlog(tmpstring,l_logfile,l_verbose)
      }
      if (iaxis == 3 && naxis[3] != 3) {
        tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//" "//str(naxis[3])//".\nThe expected numbers are [320,240,3]"
        status=status+1
        printlog(tmpstring,l_logfile,l_verbose)
      }
      if (iaxis == 4) {
        if (modeflag == 2 || modeflag == 1) {
          if (naxis[3] != 2  || naxis[4] != savesets) {
            tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//" "//str(naxis[3])//" "//str(naxis[4])//".\nThe expected numbers are [320,240,2,"//savesets//"]"
            status=status+1
            printlog(tmpstring,l_logfile,l_verbose)
          }
        }
        if (modeflag == 3 || modeflag == 4) {
          if (naxis[3] != 1  || naxis[4] != savesets) {
            tmpstring="Image extension "//str(iaxis)//" has bad image dimensions: "//str(naxis[1])//" "//str(naxis[2])//" "//str(naxis[3])//" "//str(naxis[4])//".\nThe expected numbers are [320,240,1,"//savesets//"]"
            status=status+1
            printlog(tmpstring,l_logfile,l_verbose)
          }
        }
      }
    }
  }
  else {
    tmpstring="Image extension "//str(i)//" was not opened successfully.\nThe expected number of extensions is "//str(nextends)//"."
    printlog(tmpstring,l_logfile,l_verbose)
    status=1
    bye
  }
}

if (status == 0) {
  printlog("File "//l_image//" appears to be correct.",l_logfile,verbose-)
}
else {
  printlog("File "//l_image//" has some structural errors.",l_logfile,verbose=l_verbose)
}

date | scan(l_struct)

tmpstring="Finished TCHECKSTRUCTURE. "//l_struct
printlog(tmpstring,l_logfile,verbose-)
printlog("----------------------------------------------------------------------------", logfile=l_logfile, verbose-)
printlog(" ",logfile=l_logfile, verbose-)
end
