# Copyright(c) 2001-2003 Association of Universities for Research in Astronomy, Inc.

procedure nsappwave(inimages)

# Put an approximate wavelength system in headers of NIRI spectra
#
# Version  Sept 20, 2002 JJ v1.4 release
#          Aug 19, 2003  KL IRAF2.12 - new parameter, addonly, in hedit

char    inimages    {prompt="Input raw NIRI spectra"}
char    outspectra  {"",prompt="Output spectra"}
char    prefix      {"w",prompt="Prefix for output spectra"}
char    nsappwavedb {"niri$nsappwave.dat", prompt="NSAPPWAVE Database"}
real    crval       {INDEF,prompt="Central wavelength"}
real    cdelt       {INDEF,prompt="Resolution in wavelength per pixel"}
char    key_filter  {"FILTER",prompt="Keyword for filter"}
char    key_fpmask  {"FPMASK",prompt="Keyword for filter"}
char    key_camera  {"CAMERA",prompt="Keyword for filter"}
char    key_dispaxis {"DISPAXIS",prompt="Header keyword for dispersion axis"}
char    sci_ext     {"SCI",prompt="Name or number of science extension"}
char    var_ext     {"VAR",prompt="Name or number of science extension"}
char    dq_ext      {"DQ",prompt="Name or number of science extension"}
char    key_xoff    {"XOFFSET",prompt="Header keyword for instrument x offset (in arcsec)"}
char    key_yoff    {"YOFFSET",prompt="Header keyword for instrument y offset (in arcsec)"}
char    logfile     {"",prompt="Logfile"}
bool    verbose     {yes,prompt="Verbose"}
int     status      {0,prompt="Exit status (0=good)"}
struct  *scanfile   {prompt="Internal use"}

begin

char      l_inimages, l_outspectra, l_prefix, l_nsappwavedb 
char      l_key_filter,l_key_fpmask,l_key_camera, l_key_dispaxis
char      l_logfile, l_sci_ext, l_var_ext, l_dq_ext
char      l_key_xoff, l_key_yoff
real      l_crval,l_cdelt
bool      l_verbose

char      imgin0, imgout0, l_intemp, l_outtemp, keywords, img
bool      uselut
real      wv, dw, center
int       spatial, nin, nout, nbad, nfiles, count, l_dispaxis
struct    l_date, message
char      tmpinlist, tmpoutlist, tmpin, l_temp, tmptarg, tmpoffset
char      imgoutsci, imgoutvar, imgoutdq, tmpfile1
   
# set local variables
l_inimages=inimages; l_outspectra=outspectra; l_prefix=prefix
l_nsappwavedb = nsappwavedb; l_key_filter=key_filter ; l_key_fpmask = key_fpmask
l_key_camera=key_camera; l_key_dispaxis=key_dispaxis; l_sci_ext = sci_ext
l_verbose = verbose ; l_logfile = logfile; l_crval = crval; l_cdelt=cdelt 
l_var_ext = var_ext ; l_dq_ext = dq_ext
l_key_xoff = key_xoff; l_key_yoff = key_yoff

status = 0
cache("niri","imgets")

#-------------------------------------------------------------------------
# Check the logfile
print(l_logfile) | scan(l_logfile)
if(l_logfile=="") {
   l_logfile=niri.logfile
   print(l_logfile) | scan(l_logfile)
   if(l_logfile=="") {
      l_logfile="niri.log"
      printlog("WARNING - NSAPPWAVE: Both nsappwave.logfile and niri.logfile \
         are empty.",l_logfile,verbose+)
      printlog("                     Using default file niri.log.",
         l_logfile,verbose+)
   }
}

# Make temporary files
tmpin = mktemp("tmpin")
tmpinlist = mktemp("tmpinl")
tmpoutlist = mktemp("tmpout")
tmpfile1 = mktemp("tmpfile1")

# Write to the log file
date | scan(l_date)
printlog("-----------------------------------------------------------------\
-------------",logfile=l_logfile, verbose=l_verbose)
printlog("NSAPPWAVE -- "//l_date, logfile=l_logfile, verbose=l_verbose)
printlog(" ",logfile=l_logfile, verbose=l_verbose)

#-------------------------------------------------------------------------
# Make lists of input/output spectra
# Put all input images in a temporary file: tmpfile1
if(substr(l_inimages,1,1)=="@") {
     l_temp=substr(l_inimages,2,strlen(l_inimages))
     if(!access(l_temp)) {
          printlog("ERROR - NSAPPWAVE:  Input file "//l_temp//" not found.",
               l_logfile,verbose+)
          status=1
          goto clean
     }
}
files(l_inimages,sort-, > tmpfile1)

nbad=0
nfiles=0
scanfile = tmpfile1
while(fscan(scanfile,img) != EOF) {
     gimverify(img)
     img = gimverify.outname
     if(gimverify.status==1) {
          printlog("ERROR - NSAPPWAVE: File "//img//" not found.",
               l_logfile,verbose+)
          nbad+=1
     } else if(gimverify.status>1) {
          printlog("ERROR - NSAPPWAVE: File "//img//" not a MEF FITS image.",
               l_logfile,verbose+)
          nbad+=1
     } else {
          imgets(img//"[0]","NPREPARE", >& "dev$null")
          if (imgets.value == "0") {
               printlog("ERROR - NSAPPWAVE: Image "//img//" not NPREPAREd.",
                    l_logfile,verbose+)
               nbad+=1
          } else {
            imgets(img//"[0]","NSCUT", >& "dev$null")
            if (imgets.value == "0") {
              printlog("ERROR - NSAPPWAVE: Spectrum in "//img//" not extracted using NSCUT.",
                    l_logfile,verbose+)
              nbad+=1
            }         
          }
     }
  
     nfiles+=1
     print(img, >> tmpinlist)
} # end while loop

# Check for empty file list
if(nfiles==0) {
     printlog("ERROR - NSAPPWAVE:  No input images meet wildcard criteria.",
          l_logfile,verbose+)
     status=1
     goto clean
}

# Exit if problems found with input files
if (nbad > 0) {
     printlog("ERROR - NSAPPWAVE: "//nbad//" image(s) either do not exist, \
          are not MEF files, or",l_logfile, verbose+)
     printlog("                 have not been run through NPREPARE or NSCUT.",
          l_logfile,verbose+)
     status=1
     goto clean
} # end if (nbad > 0)
delete(tmpfile1, ver-)

#-----------------------------------------------------------------------
# Now for the output images
# outspectra could contain legal * if it is of a form like %st%stX%*.fits
if(substr(l_outspectra,1,1)=="@") {
     img=substr(l_outspectra,2,strlen(l_outspectra))
     if(!access(img)) {
          printlog("ERROR - NSAPPWAVE:  Input file "//img//" not found.",
               l_logfile,verbose+)
          status=1
          goto clean
     }
}
files(l_outspectra,sort-, > tmpfile1)

# If tmpfile1 is empty, the output files names should be created with the
# prefix parameter
count=0; count(tmpfile1) | scan(count)
if(count==0) {
     delete(tmpfile1,ver-)
     scanfile=tmpinlist
     while(fscan(scanfile,img) != EOF) {
          if(l_prefix!="") print(l_prefix//img, >> tmpfile1)
          else {
               printlog("ERROR - NSAPPWAVE: no output filename.",l_logfile,
                    verbose+)
               status=1
               goto clean
          }                                   
     }
}               

# Verify that the output images do not exist
nbad=0
nfiles=0
scanfile = tmpfile1
while(fscan(scanfile,img) != EOF) {
     gimverify(img)
     img = gimverify.outname
     if(gimverify.status!=1) {
          printlog("ERROR - NSAPPWAVE: File "//img//" already exists.",
               l_logfile,verbose+)
          nbad+=1
     } 
     nfiles+=1
     print(img, >> tmpoutlist)
} # end while loop

# Exit if problems found with output files
if (nbad > 0) {
     status=1
     goto clean
} # end if (nbad > 0)
delete(tmpfile1, ver-)

# check number of input output images
count(tmpinlist) | scan(nin)
count(tmpoutlist) | scan(nout)
if(nin!=nout) {
     printlog("ERROR - NSAPPWAVE: Different number of input and output \
          spectra",l_logfile,yes)
     status=1
     goto clean
}

#----------------------------------------------------------------------
# Verify access to databases
uselut = yes
if(!access(l_nsappwavedb)) {
     printlog("WARNING - NSAPPWAVE: Cannot find database "//l_nsappwavedb,
          l_logfile,verbose+)
     if(l_cdelt!=INDEF && l_cdelt>0. && l_crval!=INDEF && l_crval>0.) {
          printlog("        Using crval ("//l_crval//") and \
               cdelt ("//l_cdelt//")",l_logfile,l_verbose)
     } else {
          printlog("ERROR - NSAPPWAVE: Cannot get crval and cdelt input \
               parameters",l_logfile,verbose+)
          status=1
          goto clean
     }
     uselut = no
}
#----------------------------------------------------------------------
# Place input and output files in tempin
# Scan through the list and do the job

printlog("          input file   -->          output file",
  l_logfile,l_verbose)
printlog("             FILTER+FPMASK+CAMERA+SPECSEC      Cent wave (A)   \
   Disp (A/pix)",
  l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)

join(tmpinlist,tmpoutlist,delim=" ",missing="Missing",
   maxchar=161,shortest=yes,verbose=l_verbose, > tmpin)
scanfile=tmpin
while(fscan(scanfile,l_intemp,l_outtemp) != EOF) {
     imgin0=l_intemp//"[0]"
     imgout0=l_outtemp//"[0]"
     imgoutsci=l_outtemp//"["//l_sci_ext//"]"
     imgoutvar=l_outtemp//"["//l_var_ext//"]"
     imgoutdq=l_outtemp//"["//l_dq_ext//"]"
     copy(l_intemp//".fits",l_outtemp//".fits",ver-)
     imgets(imgout0,l_key_dispaxis)
     l_dispaxis=int(imgets.value)
     spatial = 1
     if(l_dispaxis==1) spatial=2
     imgets(imgout0,l_key_filter)
     keywords = imgets.value
     imgets(imgout0,l_key_fpmask)
     keywords = keywords//imgets.value
     imgets(imgout0,l_key_camera)
     keywords = keywords//imgets.value
     imgets(imgout0,"NSCUTSPC")
     keywords = keywords//imgets.value

     # Search for it in the database if it exists, otherwise, use crval 
     # and cdelt
     if(uselut) {
          wv = 0.;dw=-1.
          match(keywords,l_nsappwavedb,stop=no,print_file+,
               metacharac+) | scan(wv,dw)
          if(wv==0. && dw==-1.) {
               printlog("WARNING - NSAPPWAVE: Header keywords of "//imgin0//" \
                    do not correspond to a NIRI spectroscopic setup.",
                    l_logfile,verbose+)
               if(l_crval!=INDEF && l_crval>0. && l_cdelt!=INDEF && l_cdelt>0.)
                    printlog("         Using crval ("//l_crval//") and \
                         cdelt ("//l_cdelt//")",l_logfile,l_verbose)
               else {
                    printlog("WARNING - NSAPPWAVE: Unable to get wavelength \
                         information from database or input parameters",
                         l_logfile,verbose+)
                    printlog("WARNING - NSAPPWAVE: Header not updated",
                         l_logfile,verbose+)
               }
          } else {
               l_crval = wv
               l_cdelt = 1./dw
          }        
     }  

     if(l_crval==INDEF || l_cdelt==INDEF) {
          printlog("ERROR - NSAPPWAVE: Unknown central wavelength and/or \
               resolution.",l_logfile,verbose+)
          status=1
          imdelete(l_outtemp,ver-)
          goto clean    
     }

     printf("%20s   --> %20s \n",l_intemp,l_outtemp) | scan(message)
     printlog(message,l_logfile,l_verbose)
     printf("   %38s        %8.1f        %6.3f \n",keywords,l_crval,
        l_cdelt) | scan(message)
     printlog(message,l_logfile,l_verbose)
     
     # Get the pixel position in the middle of the array
     imgets(l_outtemp//"["//l_sci_ext//"]","i_naxis"//l_dispaxis) 
     center = int(imgets.value)/2. 

# Update headers
     gemhedit(imgout0,"NSAPPWAV",l_date,"Time stamp for NSAPPWAVE")
     hedit(imgout0,"GEM-TLM",l_date,add-,addonly-,delete-,verify-,show-,update+)

     # delete cross terms from WCS
     hedit(imgoutsci,"CD"//l_dispaxis//"_"//spatial,0,add-,addonly-,delete+,
        verify-,show-,update+)
     hedit(imgoutsci,"CD"//spatial//"_"//l_dispaxis,0,add-,addonly-,delete+,
        verify-,show-,update+)

     gemhedit(imgoutsci,"WAT"//l_dispaxis//"_000","system=world","")
     gemhedit(imgoutsci,"WAT"//l_dispaxis//"_001","wtype=linear","")
     gemhedit(imgoutsci,"WAT"//l_dispaxis//"_002","units=Angstroms label=Wavelength","")
     gemhedit(imgoutsci,"CTYPE"//l_dispaxis,"LINEAR","coordinate type for the dispersion axis")
     gemhedit(imgoutsci,"CRPIX"//l_dispaxis,center,"coordinate of spectral reference pixel")
     gemhedit(imgoutsci,"CRVAL"//l_dispaxis,l_crval,"wavelength at spectral reference pixel")
     gemhedit(imgoutsci,"CD"//l_dispaxis//"_"//l_dispaxis,l_cdelt,"dispersion (Angstrom/pix)")

     gemhedit(imgoutsci,"WAT"//spatial//"_000","system=physical","")
     gemhedit(imgoutsci,"WAT"//spatial//"_001","wtype=linear","")
     gemhedit(imgoutsci,"CTYPE"//spatial,"LINEAR","coordinate type for the spatial axis")
     gemhedit(imgoutsci,"CRPIX"//spatial,1.,"coordinate of the spatial reference pixel")
     gemhedit(imgoutsci,"CRVAL"//spatial,1.,"spatial axis value at spatial reference pixel")
     gemhedit(imgoutsci,"CD"//spatial//"_"//spatial,1.,"partial of spatial axis wrt spatial dim")

     gemhedit(imgoutsci,"DC-FLAG",0,"")     
     gemhedit(imgoutsci,"DISPAXIS",l_dispaxis,"Dispersion axis")

     if(imaccess(l_outtemp//"["//l_var_ext//"]")) {
       hedit(imgoutvar,"CD"//l_dispaxis//"_"//spatial,0,add-,addonly-,delete+,
          verify-,show-,update+)
       hedit(imgoutvar,"CD"//spatial//"_"//l_dispaxis,0,add-,addonly-,delete+,
          verif-,show-,update+)
       gemhedit(imgoutvar,"WAT"//l_dispaxis//"_000","system=world","")
       gemhedit(imgoutvar,"WAT"//l_dispaxis//"_001","wtype=linear","")
       gemhedit(imgoutvar,"WAT"//l_dispaxis//"_002","units=Angstroms label=Wavelength","")
       gemhedit(imgoutvar,"CTYPE"//l_dispaxis,"LINEAR","coordinate type for the dispersion axis")
       gemhedit(imgoutvar,"CRPIX"//l_dispaxis,center,"coordinate of spectral reference pixel")
       gemhedit(imgoutvar,"CRVAL"//l_dispaxis,l_crval,"wavelength at spectral reference pixel")
       gemhedit(imgoutvar,"CD"//l_dispaxis//"_"//l_dispaxis,l_cdelt,"dispersion (Angstrom/pix)")
       gemhedit(imgoutvar,"WAT"//spatial//"_000","system=physical","")
       gemhedit(imgoutvar,"WAT"//spatial//"_001","wtype=linear","")
       gemhedit(imgoutvar,"CTYPE"//spatial,"LINEAR","coordinate type for the spatial axis")
       gemhedit(imgoutvar,"CRPIX"//spatial,1.,"coordinate of the spatial reference pixel")
       gemhedit(imgoutvar,"CRVAL"//spatial,1.,"spatial axis value at spatial reference pixel")
       gemhedit(imgoutvar,"CD"//spatial//"_"//spatial,1.,"partial of spatial axis wrt spatial dim")
       gemhedit(imgoutvar,"DC-FLAG",0,"")     
       gemhedit(imgoutvar,"DISPAXIS",l_dispaxis,"Dispersion axis")
     }

     if(imaccess(l_outtemp//"["//l_dq_ext//"]")) {
       hedit(imgoutdq,"CD"//l_dispaxis//"_"//spatial,0,add-,addonly-,delete+,
          verify-,show-,update+)
       hedit(imgoutdq,"CD"//spatial//"_"//l_dispaxis,0,add-,addonly-,delete+,
          verify-,show-,update+)
       gemhedit(imgoutdq,"WAT"//l_dispaxis//"_000","system=world","")
       gemhedit(imgoutdq,"WAT"//l_dispaxis//"_001","wtype=linear","")
       gemhedit(imgoutdq,"WAT"//l_dispaxis//"_002","units=Angstroms label=Wavelength","")
       gemhedit(imgoutdq,"CTYPE"//l_dispaxis,"LINEAR","coordinate type for the dispersion axis")
       gemhedit(imgoutdq,"CRPIX"//l_dispaxis,center,"coordinate of spectral reference pixel")
       gemhedit(imgoutdq,"CRVAL"//l_dispaxis,l_crval,"wavelength at spectral reference pixel")
       gemhedit(imgoutdq,"CD"//l_dispaxis//"_"//l_dispaxis,l_cdelt,"dispersion (Angstrom/pix)")
       gemhedit(imgoutdq,"WAT"//spatial//"_000","system=physical","")
       gemhedit(imgoutdq,"WAT"//spatial//"_001","wtype=linear","")
       gemhedit(imgoutdq,"CTYPE"//spatial,"LINEAR","coordinate type for the spatial axis")
       gemhedit(imgoutdq,"CRPIX"//spatial,1.,"coordinate of the spatial reference pixel")
       gemhedit(imgoutdq,"CRVAL"//spatial,1.,"spatial axis value at spatial reference pixel")
       gemhedit(imgoutdq,"CD"//spatial//"_"//spatial,1.,"partial of spatial axis wrt spatial dim")
       gemhedit(imgoutdq,"DC-FLAG",0,"")     
       gemhedit(imgoutdq,"DISPAXIS",l_dispaxis,"Dispersion axis")
     }

} # end of main while loop

#----------------------------------------------------------------------
clean:

scanfile=""
delete(tmpin,verify-, >& "dev$null")
delete(tmpinlist,verify-, >& "dev$null")
delete(tmpoutlist,verify-, >& "dev$null")
delete(tmpfile1,verify-, >& "dev$null")

printlog("",l_logfile,l_verbose)
if(status==0)
     printlog("NSAPPWAVE Exit status good",l_logfile,l_verbose)

printlog("------------------------------------------------------------------\
     ------------",l_logfile,l_verbose)

end
