# Copyright(c) 2000-2003 Association of Universities for Research in Astronomy, Inc.

procedure nsreduce(inimages)

# Basic reductions of NIRI spectroscopic images.
# Requires a flat, bad pixel mask, sky and/or dark image to subtract.  
# Sky images can be constructed from the input images at different nod
# positions. 
#
# A log of the reductions is written in the logfile
# The header of the reduced image will contain information about
# the reduction as a series of header entries.
#
# Variance handling:
#   var = (df/da)^2 * siga^2 + (df/db)^2 * sigb^2
#   f=a-k*b   var(f) = var(a) + k*k*var(b)
#   f=a/b     var(f) = [sci(a)^2/sci(b)^4]*var(b) + [1/sci(b)^2]*var(a)
#   f=(a-b)/c var(f) = [(sci(a)^2+sci(b)^2-2sci(a)sci(b))/sci(c)^4]*var(c) +
#			(1/sci(c)^2)*[var(a)+var(b)]	
#
# Data Quality Array handling:
#   DQ = dq(a) OR dq(b)
#
# Read noise header keyword in output image is the quadrature sum of the
# values in the input images and the sky or dark images.  No adjustment
# has been made for division by the flat field, assuming the read noise in
# the flat field is negligible.  The saturation header keyword in the 
# the output image is the input value minus the median value in the sky
# or dark image + the constant sky value added back on after.
#
# Version  Sept 20, 2002 JJ v1.4 release
#          Aug 19, 2003  KL IRAF2.12 - new parameters
#                              hedit: addonly
#                              imstat: nclip,lsigma,usigma,cache

char  inimages     {prompt="Input NIRI image(s)"}
char  outimages    {"",prompt="Output image(s)"}
char  outprefix    {"r",prompt="Prefix for output image(s)"}
char  logfile      {"",prompt="Logfile"}
bool  fl_sky       {yes,prompt="Do sky subtraction?"}
char  skyimages    {"default",prompt="Sky image(s) from other nod positions to subtract"}
real  skyrange     {30.,prompt="Time range (min) for subtracting sky if skyimages=default"}
real  nodsize      {3.,prompt="Minimum separation of nod positions in arcsec"}
bool  fl_dark      {no,prompt="Do explicit dark subtraction?"}
char  darkimage    {"",prompt="Dark current image to subtract"}
bool  fl_flat      {yes,prompt="Do flat-fielding?"}
char  flatimage    {"",prompt="Spectral flat field image to divide"}
char  statsec      {"[*,*]",prompt="Statistics section"}
char  combtype     {"median",enum="average|median",prompt="Type of combine operation for sky"}
char  rejtype      {"avsigclip",enum="none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip",prompt="Type of rejection for combining sky"}
char  scale        {"none",prompt="Image scaling for combining sky"}
char  zero         {"median",prompt="Image zeropoint offset for combining sky"}
char  weight       {"none",prompt="Image weights for combining sky"}
char  key_exptime  {"EXPTIME",prompt="Exposure time header keyword"}
real  lthreshold   {INDEF,prompt="Lower threshold"}
real  hthreshold   {INDEF,prompt="Upper threshold"}
int   nlow         {1,min=0,prompt="minmax: Number of low pixels to reject"}
int   nhigh        {1,min=0,prompt="minmax: Number of high pixels to reject"}
int   nkeep        {0,min=0,prompt="Minimum to keep or maximum to reject"}  
bool  mclip        {yes,prompt="Use median in sigma clipping algorithms?"}
real  lsigma       {3.,min=0.,prompt="Lower sigma clipping factor"}
real  hsigma       {3.,min=0.,prompt="Upper sigma clipping factor"}
char  snoise       {"0.0",prompt="ccdclip: Sensitivity noise (electrons)"}
real  sigscale     {0.1,min=0.,prompt="Tolerance for sigma clipping scaling correction"}
real  pclip        {-0.5,prompt="pclip: Percentile clipping parameter"}
real  grow         {0.0,min=0.,prompt="Radius (pixels) for neighbor rejection"}
char  sci_ext      {"SCI",prompt="Name or number of science extension"}
char  var_ext      {"VAR",prompt="Name or number of variance extension"}
char  dq_ext       {"DQ",prompt="Name or number of data quality extension"}
char  key_filter   {"FILTER",prompt="Keyword for filter ID"}
char  key_fpmask   {"FPMASK",prompt="Keyword for focal plane mask (slit) ID"}
char  key_ron      {"RDNOISE",prompt="Header keyword for read noise (e-)"}
char  key_gain     {"GAIN",prompt="New header keyword for gain (e-/ADU)"}
char  key_sat      {"SATURATI",prompt="Keyword for saturation (ADU)"}
char  key_nonlinear {"NONLINEA",prompt="Header keyword for non-linear regime (ADU)"}
char  key_xoff     {"GXOFF",prompt="Header keyword for instrument x offset (in arcsec)"}
char  key_yoff     {"GYOFF",prompt="Header keyword for instrument y offset (in arcsec)"}
char  key_time     {"UT",prompt="Header keyword for time of observation"}
bool  fl_vardq     {yes,prompt="Create variance and data quality frames?"}
bool  verbose      {yes,prompt="Verbose"}
int   status       {0,prompt="Exit status (0=good)"}
struct *scanfile   {prompt="Internal use"}

begin

char l_inimages, l_outimages, l_skyimages, l_darkimage, l_flatimage, l_filter
char l_expression, l_varexpression, l_prefix, l_logfile, l_temp, l_slit
char in[100], sky[100], out[100], skyim
char l_statsec, l_flatfilter, l_flatslit, l_flatim, l_skyfilter, l_skyslit
char l_key_filter, l_key_fpmask, l_sci_ext, l_var_ext, l_dq_ext, l_key_time
char l_key_xoff, l_key_yoff, tmpdq, tmpfile, tmplist 
char l_dqlist, l_varlist, l_scilist
int  i, nimages, nskyimages, noutimages, maxfiles, ndq
real l_skyrange, l_nodsize
bool l_fl_sky, l_fl_dark, l_fl_flat, l_verbose, l_fl_skytemp, l_fl_flattemp
bool l_fl_first, l_fl_darktemp, l_fl_vardq, bad
bool l_fl_indvardq[100], l_fl_indskyvardq[100], skyonthefly
real l_midsky, l_middark, l_mid, l_sig, l_ron, l_nonlinear
real l_middata
char l_key_ron, l_key_gain, l_key_sat, l_key_nonlinear
int  l_sat, nbad, nnew
char ima,imb,imc,imd,ime,imf,img,imh
char targlist,skylist[100]
char l_combtype, l_rejtype, l_scale, l_zero, l_weight, l_key_exptime, l_snoise
real l_lthreshold, l_hthreshold, l_lsigma, l_hsigma, l_sigscale, l_pclip, l_grow 
int  l_nlow, l_nhigh, l_nkeep
bool l_mclip
struct l_struct

status=0
nimages=0
maxfiles=100
tmpfile = mktemp("tmpin")
tmplist = mktemp("tmplist")
tmpdq = mktemp("tmpdq")
targlist = mktemp("tmptarglist")
l_filter="none"
l_slit="none"
l_skyfilter="none"
l_skyslit="none"

cache("imgets")

# set the local variables
l_inimages=inimages ; l_outimages=outimages ; l_skyimages=skyimages
l_darkimage=darkimage
l_fl_sky=fl_sky ; l_fl_dark=fl_dark ; l_fl_flat=fl_flat
l_verbose=verbose ; l_prefix=outprefix
l_logfile=logfile
l_key_filter=key_filter ; l_key_fpmask=key_fpmask ; l_flatimage=flatimage
l_sci_ext=sci_ext ; l_var_ext=var_ext ; l_dq_ext=dq_ext
l_key_ron=key_ron ; l_key_gain=key_gain ; l_key_sat=key_sat
l_fl_vardq=fl_vardq ; l_statsec=statsec ; l_key_nonlinear=key_nonlinear
l_skyrange=skyrange ; l_nodsize=nodsize
l_key_xoff=key_xoff ; l_key_yoff=key_yoff
l_combtype=combtype ; l_rejtype=rejtype ; l_scale=scale ; l_zero=zero
l_weight=weight ; l_key_exptime=key_exptime
l_lthreshold=lthreshold ; l_hthreshold=hthreshold ; l_nlow=nlow
l_nhigh=nhigh ; l_nkeep=nkeep ; l_mclip=mclip ; l_lsigma=lsigma
l_hsigma=hsigma ; l_snoise=snoise ; l_sigscale=sigscale
l_pclip=pclip ; l_grow=grow ; l_key_time=key_time

#------------------------------------------------------------------------
# Check for package log file or user-defined log file
cache("niri")
print(l_logfile) | scan(l_logfile)
if(l_logfile=="" || l_logfile==" ") {
   l_logfile=niri.logfile
   print(l_logfile) | scan(l_logfile)
   if(l_logfile=="" || l_logfile==" ") {
      l_logfile="niri.log"
      printlog("WARNING - NSREDUCE: Both nsreduce.logfile and niri.logfile are empty.",l_logfile, verbose+)
      printlog("                    Using default file niri.log.",
         l_logfile, verbose+)
   }
}
# Open log file
date | scan(l_struct)
printlog("----------------------------------------------------------------------------",l_logfile, l_verbose)
printlog("NSREDUCE -- "//l_struct, l_logfile, l_verbose)
printlog(" ",l_logfile, l_verbose)

#----------------------------------------------------------------------
# Check to see if both dark subtraction and sky subtraction are indicated
if(l_fl_sky && l_fl_dark) { 
  printlog("WARNING - NSREDUCE: Both a sky frame and a dark frame will be subtracted.",l_logfile,verbose+)
  printlog("                    Since most sky frames contain dark current, this is ",l_logfile,verbose+)
  printlog("                    probably a bad idea!  Continuing anyway.",l_logfile,verbose+)
}

# Check to see if sci_ext is defined
print(l_sci_ext) | scan(l_sci_ext)
if(l_sci_ext=="" || l_sci_ext==" ") {
     printlog("ERROR - NSREDUCE:  Extension sci_ext is undefined.",l_logfile,verbose+)
     status=1
     goto clean
}
#-----------------------------------------------------------------------
# Load up arrays of input name lists

if(substr(l_inimages,1,1)=="@") {
  l_temp=substr(l_inimages,2,strlen(l_inimages))
  if(!access(l_temp)) {
    printlog("ERROR - NSREDUCE:  Input file "//l_temp//" not found.",l_logfile,verbose+)
    status=1
    goto clean
  }
}

nimages=0
nbad=0
files(l_inimages,sort-, > tmpfile) 
scanfile=tmpfile

while(fscan(scanfile,l_temp) != EOF) {
  if(l_fl_vardq) l_fl_indvardq[nimages+1]=yes
  gimverify(l_temp)
  if(gimverify.status==1) {
    printlog("ERROR - NSREDUCE: File "//l_temp//" not found.",l_logfile,verbose+)
    nbad+=1
  } else if(gimverify.status>1) {
    printlog("ERROR - NSREDUCE: File "//l_temp//" not a MEF FITS image.",l_logfile,verbose+)
    nbad+=1
  } else {
    imgets(l_temp//"[0]","NSCUT", >& "dev$null")
    if (imgets.value == "0") {
      printlog("ERROR - NSREDUCE: Spectroscopic section of image "//l_temp,l_logfile,verbose+)
      printlog("                  has not been defined and extracted using NPREPARE and NSCUT.",l_logfile,verbose+)
      nbad+=1
    } else {
      if (l_fl_vardq && !imaccess(l_temp//"["//l_var_ext//"]") ) {
        printlog("WARNING - NSREDUCE: Cannot compute VAR planes because",l_logfile,verbose+)
        printlog("                    input image "//l_temp//" does not have a VAR plane.",l_logfile,verbose+)
        printlog("                    Resetting fl_vardq=no for this file",l_logfile,verbose+)
        l_fl_indvardq[nimages+1]=no
      }
      if (l_fl_vardq && !imaccess(l_temp//"["//l_dq_ext//"]") ) {
        printlog("WARNING - NSREDUCE: Cannot compute DQ planes because",l_logfile,verbose+)
        printlog("                    input image "//l_temp//" does not have a DQ plane.",l_logfile,verbose+)
        printlog("                    Resetting fl_vardq=no for this file",l_logfile,verbose+)
        l_fl_indvardq[nimages+1]=no
      }
    }
    # strip .fits if present
    if(substr(l_temp,strlen(l_temp)-4,strlen(l_temp)) == ".fits")
       l_temp=substr(l_temp,1,(strlen(l_temp)-5))
    nimages=nimages+1
    in[nimages]=l_temp 
    if(nimages>1) {
      for(j=1; j<nimages; j+=1) {
        if(in[nimages]==in[j]) {
          printlog("WARNING - NSREDUCE: Input image name "//in[nimages]//" repeated.",l_logfile,verbose+)
          printlog("                    Not including it again.",l_logfile,verbose+)
          nimages=nimages-1
        }
      }
    }
  }
} # end while

# Exit if problems found
if (nbad > 0) {
  printlog("ERROR - NSREDUCE: "//nbad//" image(s) either do not exist, are not MEF files, or",l_logfile,verbose+)
  printlog("                   have not been run through NPREPARE and NSCUT.",l_logfile,verbose+)
  status=1
  goto clean
}
if(nimages > maxfiles) {
  printlog("ERROR - NSREDUCE: Maximum number of input images exceeded ("//str(maxfiles)//")",l_logfile,verbose+ )
  status=1
  goto clean
}
if(nimages == 0) {
  printlog("ERROR - NSREDUCE: No valid input images.",l_logfile,verbose+ )
  status=1
  goto clean
}

printlog("Processing "//nimages//" file(s).",l_logfile,l_verbose)
scanfile="" ; delete(tmpfile,ver-, >>& "/dev/null")

#-----------------------------------------------------------------------
# Check input sky images, and create the (single) sky if explicitly defined

if(l_skyimages=="default" || l_skyimages=="" || l_skyimages==" ") {
  if(nimages>1) {
    skyonthefly=yes
    skyim="none"
    printlog("Generating the sky frame(s) using the other nod positions",l_logfile,l_verbose)
    printlog("(separations greater than "//(l_nodsize/2.)//" arcsec) and taken",l_logfile,l_verbose)
    printlog("within "//l_skyrange//" minutes.",l_logfile,l_verbose)
    printlog("",l_logfile,l_verbose)
    for(i=1; i<=nimages; i+=1) {
      sky[i]=in[i]//"_sky"
    }
  } else {
    printlog("WARNING - NSREDUCE: Only one input image and no sky images specified.",l_logfile,verbose+)
    printlog("                    Turning sky subtraction off.",l_logfile,verbose+)
    l_fl_sky=no
    skyonthefly=no
  }

# check and combine the explicitly-defined sky images
} else { 
  skyonthefly=no
  nskyimages=0
  nbad=0
  if(substr(l_skyimages,1,1)=="@") {
    l_temp=substr(l_skyimages,2,strlen(l_skyimages))
    if(!access(l_temp)) {
      printlog("ERROR - NSREDUCE:  Input file "//l_temp//" not found.",l_logfile,verbose+)
      status=1
      goto clean
    }
  }
  files(l_skyimages,sort-, > tmpfile) 
  scanfile=tmpfile

  while(fscan(scanfile,l_temp) != EOF) {
#    if(l_fl_vardq) l_fl_indvardq[nskyimages+1]=yes
    gimverify(l_temp)
    if(gimverify.status==1) {
      printlog("ERROR - NSREDUCE: File "//l_temp//" not found.",l_logfile,verbose+)
      nbad+=1
    } else if(gimverify.status>1) {
      printlog("ERROR - NSREDUCE: File "//l_temp//" not a MEF FITS image.",l_logfile,verbose+)
      nbad+=1
    } else {
      imgets(l_temp//"[0]","NSCUT", >& "dev$null")
      if (imgets.value == "0") {
        printlog("ERROR - NSREDUCE: Spectroscopic section of image "//l_temp,l_logfile,verbose+)
        printlog("                  has not been defined and extracted using NPREPARE and NSCUT.",l_logfile,verbose+)
        nbad+=1
      }
      # strip .fits if present
      if(substr(l_temp,strlen(l_temp)-4,strlen(l_temp)) == ".fits")
         l_temp=substr(l_temp,1,(strlen(l_temp)-5))
      nskyimages=nskyimages+1
      sky[nskyimages]=l_temp 
      if(nskyimages>1) {
        for(j=1; j<nskyimages; j+=1) {
          if(sky[nskyimages]==sky[j]) {
            printlog("WARNING - NSREDUCE: Input image name "//sky[nimages]//" repeated.",l_logfile,verbose+)
            printlog("                    Not including it again.",l_logfile,verbose+)
            nskyimages=nskyimages-1
          }
        }
      }
    }
  } # end while

# Exit if problems found
  if (nbad > 0) {
    printlog("ERROR - NSREDUCE: "//nbad//" sky image(s) either do not exist, are not MEF files, or",l_logfile,verbose+)
    printlog("                   have not been run through NPREPARE and NSCUT.",l_logfile,verbose+)
    status=1
    goto clean
  }
  if(nskyimages > maxfiles) {
    printlog("ERROR - NSREDUCE: Maximum number of input sky images exceeded ("//str(maxfiles)//")",l_logfile,verbose+ )
    status=1
    goto clean
  }
  if(nskyimages == 0) {
    printlog("WARNING - NSREDUCE: No valid input sky images.  Attempting to construct",l_logfile,verbose+ )
    printlog("                    sky from other nod positions, if any.",l_logfile,verbose+)
    skyonthefly=yes
  }

  if(nskyimages>1) {
    printlog("Combining "//nskyimages//" sky files.",l_logfile,l_verbose)
    if(imaccess(sky[1]//"_sky")) {
      printlog("WARNING - NSREDUCE: Sky image "//sky[1]//"_sky already exists.",l_logfile,verbose+)
      skyim=sky[1]//"_sky"
    } else {
      gemcombine("@"//tmpfile,sky[1]//"_sky",logfile="STDOUT",
         title="sky image produced by gemini.niri.nsreduce",
         combine=l_combtype,reject=l_rejtype,scale=l_scale,zero=l_zero,
         weight=l_weight,statsec=l_statsec,expname=l_key_exptime,
         lthreshold=l_lthreshold,hthreshold=l_hthreshold,nlow=l_nlow,
         nhigh=l_nhigh,nkeep=l_nkeep,mclip=l_mclip,lsigma=l_lsigma,
         hsigma=l_hsigma,key_ron=l_key_ron,key_gain=l_key_gain,
         snoise=l_snoise,sigscale=l_sigscale,pclip=l_pclip,grow=l_grow,
         bpm="sky[1]"//l_dq_ext,sci_ext=l_sci_ext,var_ext=l_var_ext,
         dq_ext=l_dq_ext,fl_vardq=l_fl_vardq,fl_dqprop=no,
         offsets="none",verbose=no, >& "dev$null")
      skyim=sky[1]//"_sky"
      if(gemcombine.status != 0) {
        printlog("WARNING - NSREDUCE:  Error in GEMCOMBINE.  Turning off sky subtraction.",l_logfile,l_verbose)
        skyim="none"
        l_fl_sky=no
      }
    }
  } else {
    skyim=l_skyimages
  }
  scanfile="" ; delete(tmpfile,ver-, >>& "/dev/null")

  imgets(skyim//"[0]",l_key_filter, >& "dev$null")
  if(imgets.value=="0") {
    printlog("WARNING - NSREDUCE: Can't read filter from header of sky image.",l_logfile,verbose+)
    l_skyfilter="none"
  } else {
    l_skyfilter=imgets.value
  }
  imgets(skyim//"[0]",l_key_fpmask, >& "dev$null")
  if(imgets.value=="0") {
    printlog("WARNING - NSREDUCE: Can't read slit from header of sky image.",l_logfile,verbose+)
    l_skyslit="none"
  } else {
    l_skyslit=imgets.value
  }
}

#---------------------------------------------------------------------------
# Now check the output images
# outimages could contain legal * if it is of a form like %st%stX%*.fits

noutimages=0
if(l_outimages!="" && l_outimages!=" ") {
  if(substr(l_outimages,1,1)=="@") 
    scanfile=substr(l_outimages,2,strlen(l_outimages))
  else if (stridx("*",l_outimages)>0)  {
    files(l_outimages,sort-) | 
       match(".hhd",stop+,print-,metach-, > tmpfile)
    scanfile=tmpfile
  } else {
    files(l_outimages,sort-, > tmpfile)
    scanfile=tmpfile
  }

  while(fscan(scanfile,l_temp) != EOF) {
    noutimages=noutimages+1
    if(noutimages > maxfiles) {
       printlog("ERROR - NSREDUCE: Maximum number of output images exceeded ("//str(maxfiles)//")",l_logfile,verbose+)
       status=1
       goto clean
     }
    out[noutimages]=l_temp 
    if(imaccess(out[noutimages])) {
      printlog("ERROR - NSREDUCE: Output image "//out[noutimages]//" already exists",l_logfile,verbose+)
      status=1
    }
  }
  if(status != 0) goto clean

# if there are too many or too few output images exit with error
  if(nimages!=noutimages) {
    printlog("ERROR - NSREDUCE: Number of input and output images are unequal.",l_logfile,verbose+)
    status=1
    goto clean
  }
} else { #use prefiex instead
  print(l_prefix) | scan(l_prefix)
  if(l_prefix=="" || l_prefix==" ") {
    printlog("ERROR - NSREDUCE: Neither output image name nor output prefix is defined.",l_logfile,verbose+)
    status=1
    goto clean
  }
  i=1
  nnew=1
  while(i<=nimages) {
    out[i]=l_prefix//in[i]
    for(j=1; j<i; j+=1) {
      if(out[i]==out[j]) {
        printlog("WARNING - NSREDUCE: Output image name "//out[i]//" repeated.",l_logfile,verbose+)
        printlog("                    Appending _"//nnew//" to the output file name.",l_logfile,verbose+)
        out[i]=out[i]//"_"//nnew
        nnew=nnew+1
      }
    }

    if(imaccess(out[i])) {
      printlog("ERROR - NSREDUCE: Output image "//out[i]//" already exists.",l_logfile,verbose+)
      status=1
    }
    i=i+1
  }
  if(status != 0) goto clean
}
scanfile="" ; delete(tmpfile,ver-, >>& "/dev/null")

#-------------------------------------------------------------------------
# Check for existence of flat or dark images, if needed, and check levels

# FLAT
if(l_fl_flat) {
  if(!imaccess(l_flatimage) && l_flatimage!="" && stridx(" ",l_flatimage)<=0) {
    printlog("ERROR - NSREDUCE: Flat field image "//l_flatimage//" not found.",l_logfile,verbose+)
    nbad+=1
    status=1
    goto clean
  }
  else if (l_flatimage=="" || stridx(" ",l_flatimage)>0 ) {
    printlog("ERROR - NSREDUCE: Flat field image is either an empty string or",l_logfile,verbose+)
    printlog("                  contains spaces.",l_logfile,verbose+)
    status=1
    goto clean
  }
  if(l_fl_vardq) {
    if((!imaccess(l_flatimage//"["//l_var_ext//"]")) || (!imaccess(l_flatimage//"["//l_dq_ext//"]"))) {
       printlog("WARNING - NSREDUCE: Flat field image does not contain both VAR and DQ planes.",l_logfile,verbose+)
       printlog("                    Setting fl_vardq=no and proceeding.",l_logfile,verbose+)
       l_fl_vardq=no
    }
  }    
  # remove .fits if present
  if(substr(l_flatimage,strlen(l_flatimage)-4,strlen(l_flatimage)) == ".fits")
     l_flatimage=substr(l_flatimage,1,(strlen(l_flatimage)-5))

  imgets(l_flatimage//"[0]",l_key_filter, >>& "dev$null")
  if(imgets.value=="0") {
     printlog("WARNING - NSREDUCE: Cannot read filter from flat field image.",l_logfile,verbose+)
     l_flatfilter="none"
  } else {
     l_flatfilter=imgets.value
  }   
  imgets(l_flatimage//"[0]",l_key_fpmask, >>& "dev$null")
  if(imgets.value=="0") {
     printlog("WARNING - NSREDUCE: Cannot read slit from flat field image.",l_logfile,verbose+)
     l_flatslit="none"
  } else {
     l_flatslit=imgets.value
  }   
}

# DARK
if(l_fl_dark) {
  if(!imaccess(l_darkimage)&&l_darkimage!="" && stridx(" ",l_darkimage)<=0) {
    printlog("ERROR - NSREDUCE: Dark current image "//l_darkimage//" not found.",l_logfile,verbose+)
    nbad+=1
    status=1
    goto clean
  }
  else if (l_darkimage=="" || stridx(" ",l_darkimage)>0 ) {
    printlog("ERROR - NSREDUCE: Dark current image is either an empty string or",l_logfile,verbose+)
    printlog("                   contains spaces.",l_logfile,verbose+)
    status=1
    goto clean
  }
  if(l_fl_vardq) {
    if((!imaccess(l_darkimage//"["//l_var_ext//"]")) || (!imaccess(l_darkimage//"["//l_dq_ext//"]"))) {
       printlog("WARNING - NSREDUCE: Dark image does not contain both VAR and DQ planes.",l_logfile,verbose+)
       printlog("                    Setting fl_vardq=no and proceeding.",l_logfile,verbose+)
       l_fl_vardq=no
    }
  }    
  # remove .fits if present
  if(substr(l_darkimage,strlen(l_darkimage)-4,strlen(l_darkimage)) == ".fits")
     l_darkimage=substr(l_darkimage,1,(strlen(l_darkimage)-5))

  imgets(l_darkimage//"[0]",l_key_filter, >>& "dev$null")
  if(imgets.value=="0") {
     printlog("WARNING - NSREDUCE: Cannot read filter from dark current image.",l_logfile,verbose+)
  } else {
    if(imgets.value!="blank" && imgets.value!="BLANK") {
       printlog("WARNING - NSREDUCE: dark current image not taken with filter wheel blanked.",l_logfile,verbose+)
    } 
  }

  # get median dark level
  imstat(l_darkimage//"["//l_sci_ext//"]"//l_statsec,fields="midpt,stddev",
     lower=INDEF,upper=INDEF,nclip=0,lsigma=INDEF,usigma=INDEF,binwidth=0.01,
     format-,cache-) | scan(l_mid,l_sig)
  imstat(l_darkimage//"["//l_sci_ext//"]"//l_statsec,fields="midpt",
     lower=(l_mid-2*l_sig),upper=(l_mid+2*l_sig),nclip=0,lsigma=INDEF,
     usigma=INDEF,binwidth=0.01,format-,cache-) | scan(l_mid)
  if(l_mid==INDEF) {
    printlog("ERROR - NSREDUCE: Statistics failed, possibly due to a bad statsec.",l_logfile,verbose+)
    status=1
    goto clean
  }
  l_middark=l_mid
} # end if(l_fl_dark)

#--------------------------------------------------------------------------
# Start making multiple sky images, if necessary

i=1 ; j=1
if(skyonthefly) {
  while(i<=nimages) {
    delete(tmpfile//","//targlist,verify-, >& "dev$null")
    sky[i]=in[i]//"_sky"
    l_skyimages=l_inimages
    files(l_skyimages,sort-, > tmpfile) 
    skylist[i]=mktemp("tmpskylist"//i)
    gemoffsetlist("@"//tmpfile,in[i],l_skyrange,targlist,tmplist,
       offsettype="time",key_time=l_key_time,logfile="STDOUT",
       verbose=no, >& "dev$null")
    if(gemoffsetlist.status!=0 || !access(targlist)) {
      printlog("WARNING - NSREDUCE: No files within the time range specified by skyrange.",l_logfile,verbose+)
      printlog("                    Continuing without sky subtraction.",l_logfile,verbose+)
      sky[i]="none"
      skylist[i]=""
    } else {
      delete(tmplist,verify-, >& "dev$null")
      gemoffsetlist("@"//targlist,in[i],l_nodsize/2.,tmplist,skylist[i],
         offsettype="distance",key_xoff=l_key_xoff,key_yoff=l_key_yoff,
         logfile="STDOUT",verbose=no, >& "dev$null")
      delete(tmplist,verify-, >& "dev$null")
      if(gemoffsetlist.status!=0) {
        printlog("WARNING - NSREDUCE: Error in GEMOFFSETLIST, most likely due to",l_logfile,verbose+)
        printlog("                    lack of WCS header information.  Continuing",l_logfile,verbose+)
        printlog("                    without sky subtraction.",l_logfile,verbose+)
        sky[i]="none"
        skylist[i]=""
      }
      if(!access(skylist[i])) {
        printlog("WARNING - NSREDUCE: No files in the sky list due to incorrect",l_logfile,verbose+)
        printlog("                    nod size.  Continuing without sky subtraction.",l_logfile,verbose+)
        sky[i]="none"
        skylist[i]=""
      }
    }

    # check to see if sky already exists and if it is unique 
    if(imaccess(sky[i])) {
      printlog("WARNING - NSREDUCE: Sky image "//sky[i]//" already exists.",l_logfile,verbose+)
      printlog("                    Existing sky will be used.",l_logfile,verbose+)
#      skylist[i]=""
    }
    if(i>1 && l_fl_sky) {
      for(j=1; j<i; j+=1) {
        l_temp=""
        if(access(skylist[i]) && access(skylist[j])) {
          diff(skylist[i],skylist[j]) | scan(l_temp)
          if(l_temp=="") {
            sky[i]=sky[j]
          }
        }
        j+=1        
      }
    }
    # combine sky 
      if(!imaccess(sky[i]) && skylist[i]!="") {
        gemcombine("@"//skylist[i],sky[i],logfile="STDOUT",
           title="sky image produced by gemini.niri.nsreduce",
           combine=l_combtype,reject=l_rejtype,scale=l_scale,zero=l_zero,
           weight=l_weight,statsec=l_statsec,expname=l_key_exptime,
           lthreshold=l_lthreshold,hthreshold=l_hthreshold,nlow=l_nlow,
           nhigh=l_nhigh,nkeep=l_nkeep,mclip=l_mclip,lsigma=l_lsigma,
           hsigma=l_hsigma,key_ron=l_key_ron,key_gain=l_key_gain,
           snoise=l_snoise,sigscale=l_sigscale,pclip=l_pclip,grow=l_grow,
           bpm="sky[1]"//l_dq_ext,sci_ext=l_sci_ext,var_ext=l_var_ext,
           dq_ext=l_dq_ext,fl_vardq=l_fl_vardq,fl_dqprop=no,
           verbose=no, >& "dev$null")
        if(gemcombine.status != 0) {
          printlog("WARNING - NSREDUCE:  Error in GEMCOMBINE.  Turning off all sky subtraction.",l_logfile,l_verbose)
          skyim="none"
          l_fl_sky=no
        }
      }
    i+=1
  } # end while
} # end if(skyonthefly)

#---------------------------------------------------------------------------
# Go through and do the math (MAIN LOOP)

printlog("  n            input file   -->            output file",l_logfile,l_verbose)
printlog("                    sky image         dark image              flat image",l_logfile,l_verbose)
printlog("                 grism/filter               slit",l_logfile,l_verbose)
printlog(" ",l_logfile,l_verbose)
l_fl_first=yes  # flag for some warnings
i=1
while(i<=nimages) {

  l_expression="a"
  l_varexpression="e"
  l_dqlist=""
  ndq=0

#----------------SKY----------------
# check for previous sky subtraction and turn it off if necessary

  if(l_fl_sky) {
    l_fl_skytemp=yes
    if(skyonthefly && skylist[i]=="") {
      l_fl_skytemp=no
      skyim="none"
    }
    imgets(in[i]//"[0]","SKYIMAGE",>>& "/dev/null")
    if(imgets.value != "0") {
      l_fl_skytemp=no
      skyim="none"
      printlog("WARNING - NSREDUCE: Image "//in[i]//" has already been sky-subtracted.",l_logfile,verbose+)
      printlog("                    Sky-subtraction NOT performed.",l_logfile,verbose+)
    }
    else {
      if(skyonthefly && skylist[i]!="") {
        skyim=sky[i]
        imgets(skyim//"[0]",l_key_filter,>>& "/dev/null")
        l_skyfilter=imgets.value
        imgets(skyim//"[0]",l_key_fpmask,>>& "/dev/null")
        l_skyslit=imgets.value
      } 

      if(l_fl_skytemp) {
        imgets(in[i]//"[0]",l_key_filter,>>& "/dev/null")
        l_filter=imgets.value
        if(l_filter!=l_skyfilter) {
          printlog("WARNING - NSREDUCE: The grism/filter for image "//in[i]//" does not match",l_logfile,verbose+)
          printlog("                    the sky image grism/filter.  Proceeding anyway.",l_logfile,verbose+)
        }
        imgets(in[i]//"[0]",l_key_fpmask,>>& "/dev/null")
        l_slit=imgets.value
        if(l_slit!=l_skyslit) {
          printlog("WARNING - NSREDUCE: The slit for image "//in[i]//" does not match",l_logfile,verbose+)
          printlog("                    the sky image slit.  Proceeding anyway.",l_logfile,verbose+)
        } 
        l_expression="("//l_expression//"-b)"
        l_varexpression="("//l_varexpression//"+f)"
        l_dqlist=l_dqlist+skyim//".fits["//l_dq_ext//"],"
        ndq=ndq+1
      }
    }
  } # end if(l_fl_sky)
  else {
    skyim="none"
    l_fl_skytemp=no
  }

#----------------DARK-----------------
# check for previous dark subtraction and turn it off if necessary
  if(l_fl_dark) {
    imgets(in[i]//"[0]","DARKIMAG",>>& "/dev/null")
    if(imgets.value != "0") {
      l_fl_darktemp=no
      l_darkimage="none"
      printlog("WARNING - NSREDUCE: Image "//in[i]//" has already been dark-subtracted",l_logfile,verbose+)
      printlog("                    by nireduce.  Dark-subtraction NOT performed.",l_logfile,verbose+)
    } else {
      l_fl_darktemp=yes
      l_expression="("//l_expression//"-c)"
      l_varexpression="("//l_varexpression//"+g)"
      l_dqlist=l_dqlist+l_darkimage//".fits["//l_dq_ext//"],"
      ndq=ndq+1
    }
  } else {
    l_darkimage="none"
    l_fl_darktemp=no
  }

#----------------FLAT------------------
# check for previous flat fielding, and turn it off if necessary
  l_flatim="none"

  if(l_fl_flat) {
    imgets(in[i]//"[0]","FLATIMAG",>>& "/dev/null")
#    imgets(in[i]//"["//l_sci_ext//"]","FLATIMAG",>>& "/dev/null")
    if(imgets.value != "0") {
      l_fl_flattemp=no
      l_flatim="none"
      printlog("WARNING - NSREDUCE: Image "//in[i]//" has already been flat-fielded.",l_logfile,verbose+)
      printlog("                    by nireduce.  Flat-fielding NOT performed.",l_logfile,verbose+)
    } else {
      l_fl_flattemp=yes
      imgets(in[i]//"[0]",l_key_filter,>>& "/dev/null")
      l_filter=imgets.value
      if(l_filter!=l_flatfilter) {
        printlog("WARNING - NSREDUCE: The grism/filter for image "//in[i]//" does not match",l_logfile,verbose+)
        printlog("                    the flat field grism/filter.  Proceeding anyway.",l_logfile,verbose+)
      }
      imgets(in[i]//"[0]",l_key_fpmask,>>& "/dev/null")
      l_slit=imgets.value
      if(l_slit!=l_flatslit) {
        printlog("WARNING - NSREDUCE: The slit for image "//in[i]//" does not match",l_logfile,verbose+)
        printlog("                    the flat field slit.  Proceeding anyway.",l_logfile,verbose+)
      }
      l_varexpression="("+l_varexpression+"/(d*d))+"
      l_varexpression=l_varexpression//"(h/(d*d*d*d))*("
      l_varexpression=l_varexpression//l_expression//"*"//l_expression//")"
      l_expression=l_expression//"/d"
      l_dqlist=l_dqlist+l_flatimage//".fits["//l_dq_ext//"],"
      ndq=ndq+1
      l_flatim=l_flatimage     
    } # end else
  } # end if(l_fl_flat)
  else {   # NO flat fielding
    l_fl_flattemp=no ; l_flatim="none"
  }

#-------------DO THE MATH----------------

  l_slit=substr(imgets.value,1,strlen(imgets.value)-6)
  printf("%3.0d %21s   -->  %21s \n",i,in[i],out[i]) | scan(l_struct)
  printlog(l_struct,l_logfile,l_verbose)
  printf("      %23s %18s %23s\n",skyim,l_darkimage,l_flatim) | scan(l_struct)
  printlog(l_struct,l_logfile,l_verbose)
  printf("      %23s %18s\n",l_filter,l_slit) | scan(l_struct)
  printlog(l_struct,l_logfile,l_verbose)

# Do the math, finally!  Pack up the results in MEF.

  l_scilist=in[i]//"["//l_sci_ext//"] "
  if(skyim=="none") l_scilist=l_scilist//"INDEF "
  else l_scilist=l_scilist+skyim//"["//l_sci_ext//"] "
  if(l_darkimage=="none") l_scilist=l_scilist//"INDEF "
  else l_scilist=l_scilist+l_darkimage//"["//l_sci_ext//"] "
  if(l_flatim=="none") l_scilist=l_scilist//"INDEF"
  else l_scilist=l_scilist+l_flatimage//"["//l_sci_ext//"]"

  imcopy(in[i]//"[0]",out[i],verbose-)
  ima=""; imb=""; imc=""; imd=""; ime=""; imf=""; img=""; imh=""
  print(l_scilist) | scan(ima,imb,imc,imd)
  imexpr(l_expression,out[i]//"["//l_sci_ext//",append]",
     ima,imb,imc,imd,outtype="real",verbose-)

  if(l_fl_vardq && l_fl_indvardq[i]) {

    l_dqlist=l_dqlist+in[i]//".fits["//l_dq_ext//"]"
    ndq=ndq+1

    l_varlist=in[i]//"["//l_sci_ext//"] "
    if(skyim=="none") l_varlist=l_varlist//"INDEF "
    else l_varlist=l_varlist+skyim//"["//l_sci_ext//"] "
    if(l_darkimage=="none") l_varlist=l_varlist//"INDEF "
    else l_varlist=l_varlist+l_darkimage//"["//l_sci_ext//"] "
    if(l_flatim=="none") l_varlist=l_varlist//"INDEF "
    else l_varlist=l_varlist+l_flatimage//"["//l_sci_ext//"] "
    l_varlist=l_varlist+in[i]//"["//l_var_ext//"] "
    if(skyim=="none") l_varlist=l_varlist//"INDEF "
    else l_varlist=l_varlist+skyim//"["//l_var_ext//"] "
    if(l_darkimage=="none") l_varlist=l_varlist//"INDEF "
    else l_varlist=l_varlist+l_darkimage//"["//l_var_ext//"] "
    if(l_flatim=="none") l_varlist=l_varlist//"INDEF"
    else l_varlist=l_varlist+l_flatimage//"["//l_var_ext//"]"

    ima=""; imb=""; imc=""; imd=""; ime=""; imf=""; img=""; imh=""
    print(l_varlist) | scan(ima,imb,imc,imd,ime,imf,img,imh)
    imexpr(l_varexpression,out[i]//"["//l_var_ext//",append]",
       ima,imb,imc,imd,ime,imf,img,imh,outtype="real",verbose-)

    if(ndq==4) {
      addmasks(l_dqlist,tmpdq//".pl","im1 || im2 || im3 || im4")
    } else if (ndq==3) {
      addmasks(l_dqlist,tmpdq//".pl","im1 || im2 || im3")
    } else if (ndq==2) {
      addmasks(l_dqlist,tmpdq//".pl","im1 || im2")
    } else {
      imcopy(l_dqlist,tmpdq//".pl",verbose-)
    }
    imcopy(tmpdq//".pl",out[i]//"["//l_dq_ext//",append]",verbose-)

  } #end if(l_fl_vardq)

# update the header
  date | scan(l_struct)
  gemhedit(out[i]//"[0]","NSREDUCE",l_struct,"Time stamp for NIREDUCE")
  hedit(out[i]//"[0]","GEM-TLM",l_struct, add-,addonly-,del-,ver-,show-,upd+)
  if(l_fl_skytemp)
    gemhedit(out[i]//"[0]","SKYIMAGE",skyim,"Sky image subtracted from raw data")
  if(l_fl_darktemp)
    gemhedit(out[i]//"[0]","DARKIMAG",l_darkimage,"Dark current image subtracted from raw data")
  if(l_fl_flattemp)
    gemhedit(out[i]//"[0]","FLATIMAG",l_flatimage,"Flat field image used")

   # these header keywords are no longer meaningful
   hedit(out[i]//"[0]",l_key_ron,add-,addonly-,delete+,verify-,show-,update+)
   hedit(out[i]//"[0]",l_key_sat,add-,addonly-,delete+,verify-,show-,update+)
   hedit(out[i]//"[0]",l_key_nonlinear,add-,addonly-,del+,verify-,show-,upd+)
   hedit(out[i]//"[0]",l_key_gain,add-,addonly-,delete+,verify-,show-,update+)

  i=i+1
  imdelete(tmpdq,ver-, >>& "/dev/null")

} # end the main while loop

#---------------------------------------------------------------------------
# Clean up
clean:

  printlog(" ",l_logfile,l_verbose)
  if(status==0) {
    printlog("NSREDUCE exit status:  good.",l_logfile,l_verbose)
  }
   printlog("----------------------------------------------------------------------------",l_logfile,l_verbose)

  scanfile=""
  delete(tmpfile//","//tmplist//","//targlist,ver-, >>& "/dev/null")
  imdelete(tmpdq,ver-, >>& "/dev/null")
  for(i=1; i<=nimages; i+=1) {
    delete(skylist[i],verify-, >& "dev$null")
  }

end



