# Copyright(c) 2001-2003 Association of Universities for Research in Astronomy, Inc.

procedure nsressky (inimages)

# script to remove residual sky from NIRI longslit spectra
# 
# Version Sept 20, 2002 MT,JJ v1.4 release
#         Aug 19, 2003  KL  IRAF2.12 - new parameter, addonly, in hedit

char    inimages     {"",prompt="Input NIRI spectra"}
char    outspectra   {"",prompt="Output spectra"}
char    outprefix    {"s",prompt="Prefix for output images"}
char    logfile      {"",prompt="Logfile name"}
char    sample       {"*",prompt="Sample to fit"}
char    function     {"chebyshev",prompt="Function to fit"}
int     order        {2,prompt="Order for fit"}
bool    fl_inter     {no,prompt="Fit interactively?"}
char    sci_ext      {"SCI",prompt="Name(s) or number(s) of science extension"}
char    var_ext      {"VAR",prompt="Name(s) or number(s) of variance extension"}
char    dq_ext       {"DQ",prompt="Name(s) or number(s) of data quality extension"}
bool    verbose      {yes,prompt="Verbose?"}
int     status       {0,prompt="Exit status (0=good)"}
struct* scanfile     {"",prompt="For internal use only"}

begin

char   l_inimages,l_outspectra,l_section,l_function,l_sample,l_logfile
char   l_outprefix
int    l_order
bool   l_verbose, l_fl_inter
char   filelist,inlist,outlist 
char   inimg,outimg,tmpfit
char   img,suf
char   l_sci_ext, l_var_ext, l_dq_ext
int    n,nin,nout,nbad
bool   bad, mef
struct sdate

# define local variables
l_inimages=inimages ; l_outspectra=outspectra
l_sample=sample ; l_outprefix=outprefix
l_logfile=logfile
l_function=function ; l_order=order
l_sci_ext = sci_ext ; l_dq_ext = dq_ext ; l_var_ext=var_ext
l_verbose=verbose ; l_fl_inter=fl_inter

status=0

cache("imgets","hedit","niri")

# open temp files
filelist=mktemp("tmpfilelist")
inlist=mktemp("tmpinlist")
outlist=mktemp("tmpoutlist")
tmpfit=mktemp("tmpfit")

# logfile name
if((l_logfile=="") || (l_logfile==" ")) {   
    l_logfile=niri.logfile
    if((l_logfile=="") || (l_logfile==" ")) {  
        l_logfile="niri.log"
        printlog("WARNING - NSRESSKY:  Both nsressky.logfile and niri.logfile are",l_logfile, yes) 
        printlog("                     empty.  Using niri.log.",l_logfile,yes)
    }
}

date | scan(sdate)
printlog("--------------------------------------------------------------------------------",l_logfile,verbose=l_verbose)
printlog("NSRESSKY -- "//sdate,l_logfile,verbose=l_verbose)
printlog(" ",l_logfile,verbose=l_verbose)
printlog("input = "//l_inimages,l_logfile,l_verbose)
printlog("output = "//l_outspectra,l_logfile,l_verbose)
printlog("sample = "//l_sample,l_logfile,l_verbose)
printlog("function = "//l_function,l_logfile,verbose=l_verbose)
printlog("order = "//l_order,l_logfile,verbose=l_verbose)
printlog(" ",l_logfile,l_verbose)

#check input files
if (l_inimages == "" || l_inimages == " "){
  printlog("ERROR - NSRESSKY: input not specified",l_logfile, yes)
  goto error
}

# check existence of list file
if (substr(l_inimages,1,1) == "@") {
    len=strlen(l_inimages)
    if (!access(substr(l_inimages,2,len))) {
        printlog("ERROR - NSRESSKY: "//substr(l_inimages,2,len)//" does not exist.",l_logfile,verbose+)
        goto error
    }
}

# check output files
if (substr(l_outspectra,1,1) == "@") {
    len=strlen(l_outspectra)
    if (!access(substr(l_outspectra,2,len))) {
        printlog("ERROR - NSRESSKY: "//substr(l_outspectra,2,len)//" does not exist.",l_logfile,verbose+)
        goto error
    }
}

# Verify that the extension names are not empty, otherwise exit gracefully
if(l_sci_ext=="" || l_sci_ext==" ") {
  printlog("ERROR - NSRESSKY: extension name sci_ext is missing",l_logfile, yes)
  goto error
}
if (l_var_ext=="" || l_var_ext ==" ") {
  printlog("ERROR - NSRESSKY: extension name dq_ext is missing",l_logfile, yes)
  goto error
} 
if (l_dq_ext=="" || l_dq_ext ==" ") {
  printlog("ERROR - NSRESSKY: extension name dq_ext is missing",l_logfile, yes)
  goto error
} 


files(l_inimages, sort-) | unique("STDIN", > inlist)
count(inlist) | scan(nin)
# check accessibility of images
nbad=0

# Check that all images in the list exist
scanfile=inlist
while(fscan(scanfile, img) != EOF) {
    suf = substr(img,strlen(img)-3,strlen(img))
    if (!imaccess(img)) {
        printlog("ERROR - NSRESSKY: Input file "//img//" does not exist.",l_logfile, yes)
        nbad=nbad+1
    } else if (suf==".imh" || imaccess(img//".imh")) {
        printlog("ERROR - NSRESSKY: Input file "//img//" is an OIF file.",l_logfile, yes)
        nbad=nbad+1
    } else if (suf==".hhh" || imaccess(img//".hhh")) {
        printlog("ERROR - NSRESSKY: Input file "//img//" is a GEIS file.",l_logfile, yes)
        nbad=nbad+1
    } else {
        suf = substr(img,strlen(img)-4,strlen(img))
        # Add .fits if necessary
        if (suf!=".fits" && imaccess(img//".fits")) {
            img = img//".fits"
        }
        imgets(img//"[0]","EXTEND", >& "dev$null")
        if(imgets.value == "F" ) {
            printlog("ERROR - NSRESSKY: Input file "//img//" is not a MEF file.",l_logfile, yes)
	    nbad=nbad+1
        }
    }
}
#end of while loop
scanfile=""

# Exit if problems found
if (nbad > 0) {
    printlog("ERROR - NSRESSKY: "//nbad//" image(s) either do not exist or are not MEF files.",l_logfile, yes)
    goto error
}
if(l_outspectra!="" && l_outspectra!=" ") {
  files(l_outspectra, sort-, > outlist)
  count(outlist) | scan(nout)
  if (nin != nout) {
      printlog("ERROR - NSSRESSKY: different numbers of input and output images",l_logfile,yes)
      goto error
  }
} else {
  print(l_outprefix) | scan(l_outprefix)
  if(l_outprefix=="" || l_outprefix==" ") {
    printlog("ERROR - NSRESSKY: Neither output image name nor output prefix is defined.",l_logfile,verbose+)
    goto error
  }
  scanfile=inlist
  while(fscan(scanfile, img) != EOF) {
    print(l_outprefix//img, > outlist)
  }
  scanfile=""
}

join(inlist,outlist,output=filelist,delim=" ",verbose-)
scanfile=filelist
while(fscan(scanfile, inimg, outimg) != EOF) {
  printlog(inimg//" "//outimg,l_logfile,l_verbose)
  if (!imaccess(inimg//"["//l_sci_ext//"]")) {
    printlog("ERROR - NSRESSKY: Could not access "//inimg//"["//l_sci_ext//"]",l_logfile,yes)
    goto error
  }

  fit1d(inimg//"["//l_sci_ext//"]",tmpfit,"fit",axis=2,
     function=l_function,order=l_order,low_reject=3.,high_reject=3.,
     niter=3,interactive=l_fl_inter,sample=l_sample)
	
  gimverify(outimg); outimg=gimverify.outname
  if(gimverify.status==1) {
    copy(inimg//".fits",outimg//".fits",verbose-)
    imarith(inimg//"["//l_sci_ext//"]","-",tmpfit,
       outimg//"["//l_sci_ext//",overwrite]",verbose-)
    gemhedit(outimg//"[0]","NSRESSKY",sdate,"Time stamp for NSRESSKY")
    hedit(outimg//"[0]","GEM-TLM",sdate,add+,addonly-,del-,ver-,show-,update+)
    if (access(inimg//"["//l_var_ext//"]")) {
      imarith(tmpfit,"*",tmpfit,tmpfit,verbose-)
      imarith(inimg//"["//l_var_ext//"]","+",tmpfit,
         outimg//"["//l_var_ext//",overwrite]",verbose-)
      imcopy(inimg//"["//l_dq_ext//"]",outimg//"["//l_var_ext//",overwrite]",verbose-)
    }
    imdelete(tmpfit,verify-, >& "dev$null")

  } else {
    printlog("WARNING - NSRESSKY: output image "//outimg//" already exists.",
       l_logfile,verbose+)
  }			
}
scanfile=""

# clean up
goto clean

error:
{
  status=1
  goto clean
}

clean:

delete(filelist//","//inlist//","//outlist,verify-, >& "dev$null")
#delete(scilist//","//varlist//","//dqlist//","//extlist,verify-, >& "dev$null")
imdelete(tmpfit,verify-, >& "dev$null")

printlog(" ",l_logfile,l_verbose)
printlog("NSRESSKY done",l_logfile,l_verbose)
printlog("--------------------------------------------------------------------------------",l_logfile,l_verbose )

end
