# Copyright(c) 2001-2003 Association of Universities for Research in Astronomy, Inc.

procedure nsstack(inimages)

# Version Sept 20, 2002 MT,JJ v1.4 release
#         Aug 19, 2003  KL  IRAF2.12 - new parameter, addonly, in hedit
#                           Fixed even though all hedit calls are commented out

char   inimages     {prompt="Input images or spectra to combine"}
real   tolerance    {0.5,prompt="Maximum offset for grouping (arcsec)"}
char   combtype      {"average",prompt="Combination operation",\
enum="average|median"}
char   rejtype      {"avsigclip",prompt="Rejection algorithm",\
enum="none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip"}
char   key_exptime  {"EXPTIME",prompt="Exposure time header keyword"}
char   key_xoff     {"GXOFF",prompt="Header keyword for instrument x offset (in arcsec)"}
char   key_yoff     {"GYOFF",prompt="Header keyword for instrument y offset (in arcsec)"}
char   statsec      {"[*,*]",prompt="Image section to be used for statistics"}
char   stacksuffix  {"_stack",prompt="Suffix for combined stacks"}
char   scale        {"none",prompt="Image scaling"}
char   zero         {"none",prompt="Image zeropoint offset"}
char   weight       {"none",prompt="Image weights"}
real   lthreshold   {INDEF,prompt="Lower threshold"}
real   hthreshold   {INDEF,prompt="Upper threshold"}
int    nlow         {1,min=0,prompt="minmax: Number of low pixels to reject"}
int    nhigh        {1,min=0,prompt="minmax: Number of high pixels to reject"}
int    nkeep        {0,min=0,prompt="Minimum to keep or maximum to reject"}  
bool   mclip        {yes,prompt="Use median in sigma clipping algorithms?"}
real   lsigma       {3.,prompt="Lower sigma clipping factor"}
real   hsigma       {3.,prompt="Upper sigma clipping factor"}
char   key_ron      {"RDNOISE",prompt="Keyword for readout noise in e-"}
char   key_gain     {"GAIN",prompt="Keyword for gain in electrons/ADU"}
real   ron          {0.0,min=0.,prompt="Readout noise rms in electrons"}
real   gain         {1.0,min=0.00001,prompt="Gain in e-/ADU"}
char   snoise       {"0.0",prompt="ccdclip: Sensitivity noise (electrons)"}
real   sigscale     {0.1,prompt="Tolerance for sigma clipping scaling correction"}
real   pclip        {-0.5,prompt="pclip: Percentile clipping parameter"}
real   grow         {0.0,prompt="Radius (pixels) for neighbor rejection"}
char   sci_ext      {"SCI",prompt="Name or number of science extension"}
char   var_ext      {"VAR",prompt="Name or number of variance extension"}
char   dq_ext       {"DQ",prompt="Name or number of data quality extension"}
bool   fl_vardq     {yes,prompt="Create variance and data quality frames in output images?"}
char   logfile      {"",prompt="Logfile"}
bool   verbose      {yes,prompt="Verbose output?"}
int    status       {0,prompt="Exit status (0=good)"}
struct *scanfile    {"",prompt="Internal use only"} 


begin

char  l_inimages, l_logfile, l_sci_ext, l_var_ext, l_dq_ext, l_stacksuffix
char  l_key_exptime, l_statsec, l_key_xoff, l_key_yoff
char  l_combtype, l_rejtype
bool  l_fl_vardq, l_verbose
char  l_scale, l_zero, l_weight
char  l_key_ron, l_key_gain, l_snoise
char  l_sgain, l_sron
real  l_ron, l_gain, l_lthreshold, l_hthreshold, l_lsigma, l_hsigma
real  l_grow, l_sigscale, l_pclip, l_tolerance
int   l_nlow, l_nhigh, l_nkeep
bool  l_mclip
char  img, tmpimg
char  tmpframes, tmpokframes, tmptarget, tmpoffset, tmphead
int   nbad, nfiles, nn, ntarg, noff, nstacks, ii, jj
struct sdate

l_inimages = inimages; l_logfile=logfile; l_stacksuffix=stacksuffix
l_combtype=combtype; l_rejtype=rejtype; l_scale=scale; l_zero=zero
l_weight=weight; l_lthreshold=lthreshold; l_hthreshold=hthreshold
l_nlow=nlow; l_nhigh=nhigh; l_mclip=mclip; l_lsigma=lsigma; l_hsigma=hsigma
l_key_ron=key_ron; l_key_gain=key_gain; l_ron=ron; l_gain=gain; l_snoise=snoise
l_sigscale=sigscale; l_pclip=pclip; l_grow=grow
l_tolerance=tolerance
l_sci_ext=sci_ext; l_var_ext=var_ext; l_dq_ext=dq_ext
l_key_exptime=key_exptime; l_statsec=statsec; l_nkeep=nkeep
l_fl_vardq=fl_vardq; l_verbose=verbose
l_key_xoff=key_xoff; l_key_yoff=key_yoff

status=0

#----------------------------------------------------------------------------
# Make temp files
tmpframes = mktemp("tmpframes")
tmpokframes = mktemp("tmpokframes")
tmptarget = mktemp("tmptarget")
tmpoffset = mktemp("tmpoffset")
tmphead   = mktemp("tmphead")

# Start logging to file
cache("niri")
print(l_logfile) | scan(l_logfile)
if(l_logfile=="") {
  l_logfile=niri.logfile
  print(l_logfile) | scan(l_logfile)
  if(l_logfile=="") {
    l_logfile="niri.log"
    printlog("WARNING - NSSTACK: Both nsstack.logfile and niri.logfile are empty.",l_logfile,verbose+)
    printlog("                   Using default file niri.log.",l_logfile,verbose+)
  }
}
date | scan(sdate)
printlog("----------------------------------------------------------------\
	----------------",l_logfile,verbose=l_verbose)
printlog("NSSTACK -- "//sdate,l_logfile,verbose=l_verbose)
printlog(" ",l_logfile,verbose=l_verbose)
printlog("Input images = "//l_inimages,l_logfile,l_verbose)
if(l_verbose) files(l_inimages)
files(l_inimages, >> l_logfile)
printlog("Suffix for output images = "//l_stacksuffix,l_logfile,l_verbose)
printlog("Offset tolerance    = "//l_tolerance,l_logfile,l_verbose)
printlog("Statistics section  = "//l_statsec,l_logfile,l_verbose)
printlog("Combine algorithm   = "//l_combtype,l_logfile,l_verbose)
printlog("Rejection algorithm = "//l_rejtype,l_logfile,l_verbose)

#-------------------------------------------------------------------------
# If mef extension names are empty, exit
print(l_sci_ext) | scan(l_sci_ext)
if(l_sci_ext=="") {
  printlog("ERROR - NSSTACK: Empty sci_ext extension",l_logfile,verbose+)
  status=1
  goto clean
}
if(l_fl_vardq) {
  print(l_var_ext) | scan(l_var_ext)
  if(l_var_ext=="") {
    printlog("ERROR - NSSTACK: Empty variance extension name", l_logfile,verbose+)
    status=1
    goto clean
  }
  print(l_dq_ext) | scan(l_dq_ext)
  if(l_dq_ext=="") {
    printlog("ERROR - NSSTACK: Empty data quality extension name",l_logfile,verbose+)
    status=1
    goto clean
  }
}

# Check if key_exptime is set
print(l_key_exptime) | scan(l_key_exptime)
if(l_key_exptime=="") {
  printlog("WARNING - NSSTACK: key_exptime not set, cannot check exposure times.",
     l_logfile,verbose+)
}

# Check for empty statistics section
print(l_statsec) | scan(l_statsec)
if(l_statsec=="") {
  l_statsec="[*,*]"
  printlog("WARNING - NSSTACK: No statsec defined.  Using entire image.",
     l_logfile,verbose+)
}

#--------------------------------------------------------------------------------
# Check the input frames - put all frames in file tmpokframes
files(l_inimages,sort+) | unique("STDIN", > tmpframes)
count(tmpframes) | scan(nfiles)
nbad=0
scanfile=tmpframes
while(fscan(scanfile,img) != EOF) {
  gimverify(img); img = gimverify.outname
  if(gimverify.status!=0) {
    printlog("WARNING - NSSTACK: Input image: "//img//".fits does not exist.",
       l_logfile,l_verbose)
    nbad+=1
  } else print(img, >> tmpokframes)					
}
if(nbad==nfiles) {
  printlog("ERROR - NSSTACK: No frames to combine.",l_logfile,verbose+)
  status=1
  goto clean
}
delete (tmpframes,ver-)

# If only one file to combine - exit.
count(tmpokframes) | scan(nfiles)
if(nfiles==1) {
  printlog("ERROR - NSCOMBINE: Only one image to combine",l_logfile,verbose+)
  status=1
  goto clean
}

#------------------------------------------------------------------------
# Group files with same offset in different files
nn = nfiles
ii=1
while (nn>0) {
  head(tmpokframes,nlines=1) | scan(img)
  gemoffsetlist("@"//tmpokframes,img,l_tolerance,tmptarget,tmpoffset,
     key_xoff=l_key_xoff,key_yoff=l_key_yoff,logfile=l_logfile,
     verbose=no,offsettype="distance",key_time="",>& "dev$null")
  if(gemoffsetlist.status==1) {
    status=1
    printlog("ERROR - NSSTACK: gemoffsetlist could not determine the offsets.",
       l_logfile,verbose+)
    goto clean
  }
  ntarg=0; noff=0
  count(tmptarget) | scan(ntarg)
  if(access(tmpoffset)) count(tmpoffset) | scan(noff)
  nn=nn-ntarg
  printlog(" ",l_logfile,l_verbose)
  printlog("Combining set "//ii//":",l_logfile,l_verbose)
  scanfile=tmptarget
  while(fscan(scanfile,tmpimg) != EOF) {
    printlog("  "//tmpimg,l_logfile,l_verbose)
  }
  ii+=1

# Combine frames in tmptarget	
# Only use gemcombine if more than one image, otherwise copy the image
  if(ntarg<=1) { 
    copy(img//".fits",img//l_stacksuffix//".fits",verbose-)
    gemhedit(img//l_stacksuffix//"["//l_sci_ext//"]","extver",1,"Extension version")	
#    hedit(img//l_stacksuffix//"[0]","GXOFF",add-,addonly-,del+,ver-,show-,upd+)
#    hedit(img//l_stacksuffix//"[0]","GYOFF",add-,addonly-,del+,ver-,show-,upd+)
#    hedit(img//l_stacksuffix//"[0]","GOFFREF",add-,addonly-,del+,upd+,ver-,show-)
    gemhedit(img//l_stacksuffix//"[0]","NSSTACK",sdate,"Time stamp for NSSTACK")
    gemhedit(img//l_stacksuffix//"[0]","GEM-TLM",sdate,"Last modification with GEMINI")
  } else {
    if(access(img//l_stacksuffix//".fits")) {
      printlog("WARNING - NSSTACK: existing stack ("//img//l_stacksuffix//") will be used.",l_logfile,verbose+)
    } else {	
      gemcombine("@"//tmptarget,img//l_stacksuffix,title="",logfile=l_logfile,
         combine=l_combtype,reject=l_rejtype,scale=l_scale,zero=l_zero,
         weight=l_weight,statsec=l_statsec,expname=l_key_exptime,
         lthreshold=l_lthreshold,hthreshold=l_hthreshold,nlow=l_nlow,
         nhigh=l_nhigh,nkeep=l_nkeep,mclip=l_mclip,lsigma=l_lsigma,
         hsigma=l_hsigma,key_ron=l_key_ron,key_gain=l_key_gain,
         ron=l_ron,gain=l_gain,snoise=l_snoise,sigscal=l_sigscale,
         pclip=l_pclip,grow=l_grow,bpm="",sci_ext=l_sci_ext,
         var_ext=l_var_ext,dq_ext=l_dq_ext,fl_vardq=l_fl_vardq,
         offsets="none",verbose-, >& "dev$null")

      # update headers of output images
      gemhedit(img//l_stacksuffix//"["//l_sci_ext//"]","extver",1,"Extension version")	
#      hedit(img//l_stacksuffix//"[0]","GXOFF",add-,addonly-,del+,upd+,ver-,show-)
#      hedit(img//l_stacksuffix//"[0]","GYOFF",add-,addonly-,del+,upd+,ver-,show-)
#      hedit(img//l_stacksuffix//"[0]","GOFFREF",add-,addonly-,del+,upd+,ver-,show-)
      gemhedit(img//l_stacksuffix//"[0]","GEM-TLM",sdate,"Last modification with GEMINI")

      printf("%-8s= \'%-18s\' / %-s\n","NSSTACK",sdate,"Time stamp for NSSTACK", >> tmphead)
      printf("%-8s= %20.3f / %-s\n","NSSTKTOL",l_tolerance,"Spatial tolerance for NSSTACK", >> tmphead)
      printf("%-8s= \'%-18s\' / %-s\n","NSSTKSTA",l_statsec,"Statistics section used by NSSTACK", >> tmphead)
      printf("%-8s= \'%-18s\' / %-s\n","NSSTKCOM",l_combtype,"Combine method used by NSSTACK", >> tmphead)
      if(l_zero!="none") {
         printf("%-8s= \'%-18s\' / %-s\n","NSSTKZER",l_zero,"Statistic used to compute additive offsets", >> tmphead)
      }
      if(l_scale!="none") {
         printf("%-8s= \'%-18s\' / %-s\n","NSSTKSCL",l_scale,"Statistic used to compute scale factors", >> tmphead)
      }
      if(l_weight!="none") {
         printf("%-8s= \'%-18s\' / %-s\n","NSSTKWEI",l_weight,"Statistic used to compute relative weights", >> tmphead)
      }
      if(l_rejtype!="none") {
         printf("%-8s= \'%-18s\' / %-s\n","NSSTKREJ",l_rejtype,"Rejection algorithm used by NSSTACK", >> tmphead)
         printf("%-8s= %20.5f / %-s\n","NSSTKLTH",l_lthreshold,"Lower threshold before combining", >> tmphead)
         printf("%-8s= %20.5f / %-s\n","NSSTKHTH",l_hthreshold,"Upper threshold before combining", >> tmphead)
         printf("%-8s= %20.2f / %-s\n","NSSTKGRW",l_grow,"Radius for additional pixel rejection", >> tmphead)
      }
      if(l_rejtype=="minmax") {
         printf("%-8s= %20.0f / %-s\n","NSSTKNLO",l_nlow,"Low pixels rejected (minmax)", >> tmphead)
         printf("%-8s= %20.0f / %-s\n","NSSTKNHI",l_nhigh,"High pixels rejected (minmax)", >> tmphead)
      }
      if(l_rejtype=="sigclip" || l_rejtype=="avsigclip" || l_rejtype=="ccdclip" \
           || l_rejtype=="crreject" || l_rejtype=="pclip") {
         printf("%-8s= %20.2f / %-s\n","NSSTKLSI",l_lsig,"Lower sigma for rejection", >> tmphead)
         printf("%-8s= %20.2f / %-s\n","NSSTKHSI",l_hsig,"Upper sigma for rejection", >> tmphead)
         printf("%-8s= %20.0f / %-s\n","NSSTKNKE",l_nkeep,"Min(max) number of pixels to keep(reject)", >> tmphead)
         printf("%-8s= \'%-18b\' / %-s\n","NSSTKMCL",l_mclip,"Use median in clipping algorithms", >> tmphead)
      } 
      if(l_rejtype=="sigclip" || l_rejtype=="avsigclip" || l_rejtype=="ccdclip" \
           || l_rejtype=="crreject") {
         printf("%-8s= %20.2f / %-s\n","NSSTKSSC",l_sigscale,"Tolerance for sigma clip scaling correction", >> tmphead)
     }
      if(l_rejtype=="pclip") {
         printf("%-8s= %20.2f / %-s\n","NSSTKPCL",l_pclip,"Percentile clipping factor used by pclip", >> tmphead)
      }     
      if(l_rejtype=="ccdclip") {
         printf("%-8s= \'%-18s\' / %-s\n","NSSTKSNO",l_snoise,"Sensitivity noise (e) used by ccdclip", >> tmphead)
      }     
 
      jj=1
      scanfile=tmptarget
      while(fscan(scanfile,tmpimg) != EOF) {
        printf("%-8s= \'%-18s\' / %-s\n","NSSTK000"+jj,tmpimg,"Input image", >> tmphead)
        jj+=1
      }
      # put all the new stuff in the header
      mkheader(img//l_stacksuffix//"[0]",tmphead,append+,verbose-)

      if(l_fl_vardq) {				
        gemhedit(img//l_stacksuffix//"["//l_var_ext//"]","extver",1,"Extension version")	
        gemhedit(img//l_stacksuffix//"["//l_dq_ext//"]","extver",1,"Extension version")
      }
    }	
  }
  delete(tmpokframes,ver-)
  if(access(tmpoffset)) rename(tmpoffset,tmpokframes,field="all")
  delete(tmptarget,ver-)
} # end while


clean:
   scanfile = ""
   delete(tmpframes//","//tmpokframes//","//tmptarget,verify-, >>& "dev$null")
   delete(tmpoffset//","//tmphead,verify-, >>& "dev$null")
   if(status==0) {
     printlog("",l_logfile,l_verbose)
     printlog("NSSTACK: Exit status good.",l_logfile,l_verbose)
     printlog("-------------------------------------------------------------\
             -------------------",l_logfile,l_verbose)	
   }
end
