# Copyright(c) 2001-2003 Association of Universities for Research in Astronomy, Inc.

procedure nstransform(inimages)

# Rectify NIRI spectra using available wavelength calibrations
# and S-distoritions.
#
# Version: Sept 20, 2002 IJ,MT,JJ v1.4 release
#          Aug 19, 2003  KL  IRAF2.12 - new parameter, addonly, in hedit

char  inimages      {prompt="Input NIRI spectra"}
char  outspectra    {"",prompt="Output spectra"}
char  outprefix     {"t",prompt="Prefix for output spectra"}
char  lamptransf    {"",prompt="Name of arc lamp spectrum used to determine \
                        wavelength calibration"}
char  sdisttransf   {"",prompt="Name of image used to determine \
                         S-distortion calibration"}
char  key_dispaxis  {"DISPAXIS",prompt="Header keywordd for dispersion axis"}
int   dispaxis      {1,min=1,max=2,prompt="Dispersion axis"}
char  database      {"database",prompt="Directory for calibration files"}
bool  fl_inter      {yes,prompt="Examine fitted transformations interactively"}
char  function      {"chebyshev",min="legendre|chebyshev|spline1|spline3",
                         prompt="Coordinate fitting function"}
int   xorder        {4,min=1,prompt="X order of coordinate fitting function"}
int   yorder        {4,min=1,prompt="Y order of coordinate fitting function"}
char  interptype    {"poly3",enum="nearest|linear|poly3|poly5|spline3",
                         prompt="Interpolation type for transform"}
bool  xlog          {no,prompt="Logarithmic x coordinate for transform"}
bool  ylog          {no,prompt="Logarithmic y coordinate for transform"}
char  sci_ext       {"SCI",prompt="(Root) name or number of science extension"}
char  var_ext       {"VAR",prompt="(Root) name or number of variance extension"}
char  dq_ext        {"DQ",prompt="(Root) name or number of data quality \
                         extension"}
char  logfile       {"",prompt="Logfile"}
bool  verbose       {yes,prompt="Verbose"}
int   status        {0,prompt="Exit status (0=Good)"}
struct* scanfile    {"",prompt="Internal use"}


begin

char      l_inimages, l_outspectra, l_prefix, l_lamptransf, l_sdisttransf
char      l_key_dispaxis, l_database, l_function, l_interptype
char      l_sci_ext, l_var_ext, l_dq_ext, l_logfile
bool      l_fl_inter, l_xlog, l_ylog, l_verbose 
int       l_dispaxis, l_xorder, l_yorder

char      tmpinlist, tmpoutlist, tmpin
char      tmplamps, tmpsdist, tmpdqint, s_database
int       nin, nout, nlamps, nsdist ,ii
bool      l_fl_lamptransf, l_fl_sdisttransf
char      l_fitname, l_fitname_lamp, l_fitname_sdist
struct    l_date
char      wmefimages, l_extnames
char      tmpfile1, img, imgin, imgout, lamp, sdist, sec
int       nbad, nfiles, lcount
bool      fl_runbefore

# Define temporary files
tmpfile1 = mktemp("tmpfile1")
tmpin = mktemp("tmpin")
tmpinlist = mktemp("tmpinl")
tmpoutlist = mktemp("tmpout")
tmplamps = mktemp("tmplam")
tmpsdist = mktemp("tmpsdi")

# Initialize exit status
status=0

# cache parameter files
cache("imgets","niri","hedit")

# Set the local variables
l_inimages=inimages ; l_outspectra=outspectra ; l_prefix=outprefix
l_lamptransf=lamptransf ; l_sdisttransf=sdisttransf
l_dispaxis=dispaxis ; l_key_dispaxis=key_dispaxis
l_database=database
print(l_database) | scan(l_database)
s_database=""
if(l_database!="") s_database=l_database//"/"
l_fl_inter=fl_inter
l_function=function ; l_xorder=xorder ; l_yorder=yorder
l_interptype=interptype ; l_xlog=xlog ; l_ylog=ylog
l_sci_ext=sci_ext;l_var_ext=var_ext;l_dq_ext=dq_ext
l_logfile=logfile ; l_verbose=verbose 

print(l_logfile) | scan(l_logfile)
if(l_logfile=="") {
   l_logfile=niri.logfile
   if(l_logfile=="") {
      l_logfile="niri.log"
      printlog("WARNING - NSTRANSFORM:  Both nstransform.logfile and niri.logfile",
         logfile=l_logfile, verbose+)
      printlog("                        are empty.  Using niri.log.",
         logfile=l_logfile, verbose+)
   }
}

######################################################################
# Write to the logfile
printlog("-------------------------------------------------------------------------------",l_logfile,l_verbose)
date | scan(l_date)
printlog("NSTRANSFORM -- "//l_date,l_logfile,l_verbose)
printlog("",l_logfile,l_verbose)

#####################################################################
# Make lists of input spectra
# Put all input images in a temporary file: tmpfile1

if(substr(l_inimages,1,1)=="@") {
  l_temp=substr(l_inimages,2,strlen(l_inimages))
  if(!access(l_temp)) {
    printlog("ERROR - NSTRANSFORM:  Input file "//l_temp//" not found.",
     l_logfile,verbose+)
    status=1
    goto clean
  }
}

files(l_inimages,sort-, > tmpfile1)

nbad=0; nfiles=0
scanfile = tmpfile1
while(fscan(scanfile,img) != EOF) {
  fl_runbefore=no
  gimverify(img); img=gimverify.outname
  if(gimverify.status==1) {
    printlog("ERROR - NSTRANSFORM: File "//img//" not found.",l_logfile,
       verbose+)
    nbad+=1
  } else if(gimverify.status>1) {
    printlog("ERROR - NSTRANSFORM: File "//img//" not a MEF FITS image.",
       l_logfile,verbose+)
    nbad+=1
  } else {
    imgets(img//"[0]","NPREPARE", >& "dev$null")
    if (imgets.value == "0") {
      printlog("ERROR - NSTRANSFORM: Image "//img//" not NPREPAREd.",
         l_logfile,verbose+)
      nbad+=1
    }    
    imgets(img//"[0]","NSTRANSF", >& "dev$null")
    if (imgets.value != "0") {
      printlog("WARNING - NSTRANSFORM: Image "//img//" has been \
               run through nswavelength before.",l_logfile,verbose+)
      fl_runbefore=yes
    }        
  }

  if(!fl_runbefore) {
    nfiles+=1
    print(img, >> tmpinlist)
  }      
} # end while loop

# Check for empty file list
if(nfiles==0) {
  printlog("ERROR - NSTRANSFORM:  No input images to process.",
     l_logfile,verbose+)
  status=1
  goto clean
}

# Exit if problems found with input files
if (nbad > 0) {
   printlog("ERROR - NSTRANSFORM: "//nbad//" image(s) either do not exist, \
     are not MEF files, or",l_logfile, verbose+)
   printlog("                 have not been run through NPREPARE.",l_logfile,
     verbose+)
   status=1
   goto clean
} # end if (nbad > 0)

printlog("Using input files:",l_logfile,l_verbose)
if(l_verbose) type(tmpinlist)
type(tmpinlist, >> l_logfile)
delete(tmpfile1, ver-)

#----------------------------------------------------------------------
# Now for the output images
# outspectra could contain legal * if it is of a form like %st%stX%*.fits
if(substr(l_outspectra,1,1)=="@") {
  l_temp=substr(l_outspectra,2,strlen(l_outspectra))
  if(!access(l_temp)) {
    printlog("ERROR - NSTRANSFORM:  Input file "//l_temp//" not found.",
     l_logfile,verbose+)
    status=1
    goto clean
  }
}

files(l_outspectra,sort-, > tmpfile1)

# If tmpfile1 is empty, the output files names should be created with the
# prefix parameter
lcount=0; count(tmpfile1) | scan(lcount)
if(lcount==0) {
     delete(tmpfile1,ver-)
     scanfile=tmpinlist
     while(fscan(scanfile,img) != EOF) {
          print(l_prefix//img, >> tmpfile1)
     }
}               

# Verify that the output images do not exist
nbad=0
nfiles=0
scanfile = tmpfile1
while(fscan(scanfile,img) != EOF) {
  gimverify(img)
  img = gimverify.outname
  if(gimverify.status!=1) {
     printlog("ERROR - NSTRANSFORM: File "//img//" already exists.",l_logfile,
          verbose+)
     nbad+=1
  } 
  nfiles+=1
  print(img, >> tmpoutlist)
} # end while loop

# Exit if problems found with output files
if (nbad > 0) {
   status=1
   goto clean
} # end if (nbad > 0)

printlog("Using output files:",l_logfile,l_verbose)
if(l_verbose) type(tmpoutlist)
type(tmpoutlist, >> l_logfile)
delete(tmpfile1, ver-)

#----------------------------------------------------------------------
# Check number of input output images
count(tmpinlist) | scan(nin)
count(tmpoutlist) | scan(nout)
if(nin!=nout) {
     printlog("ERROR - NSTRANSFORM: Different number of input and output \
          spectra",l_logfile,verbose+)
     status=1
     goto clean
}


printf("Science extension           : %-8d\n",l_sci_ext) | scan(l_date)
#printlog(l_date,l_logfile,l_verbose)
          
scanfile = tmpinlist
while(fscan(scanfile,img) != EOF) {
  sec = "["//l_sci_ext//"]"
  imgets(img//sec,"NSWAVELE", >& "dev$null")
  if(imgets.value != "0") {
    printlog("WARNING - NSTRANSFORM: Image "//img//" has been \
             run through nswavelength before.",l_logfile,verbose+)
    fl_runbefore=yes
  }
}     
#----------------------------------------------------------------------
# Check number of lamps and sdist calibration files. Check if they exist.

# First check the wavelength calibration files:
print(l_lamptransf) | scan(l_lamptransf)
if(l_lamptransf!="") {
     l_fl_lamptransf=yes
     if(substr(l_lamptransf,1,1)=="@") files(l_lamptransf,sort-, > tmplamps)
     else print(l_lamptransf, > tmplamps)
     count(tmplamps) | scan(nlamps)
     if(nlamps!=1 && nlamps!=nin) {     
       printlog("ERROR - NSTRANSFORM: Number of wavelength calibration \
               files is "//nlamps//". It must be either 1",l_logfile,verbose+)
       printlog("                     or equal to the number of input \
               spectra (inimages).",l_logfile,verbose+)
       status=1
       goto clean
     }
     
     scanfile=tmplamps
     while(fscan(scanfile,img) != EOF) {
       sec = "["//l_sci_ext//"]"
       if(!access(s_database//"id"//img//sec)) {
         printlog("ERROR - NSTRANSFORM: wavelength calibration \
            file "//s_database//img//sec//" does not exist.",l_logfile,verbose+)
         status=1
       }                    
     }
     if(status==1) goto clean
               
} else l_fl_lamptransf=no

# Second check the distortion correction files:
print(l_sdisttransf) | scan(l_sdisttransf)
if(l_sdisttransf!="") {
     l_fl_sdisttransf=yes
     if(substr(l_sdisttransf,1,1)=="@") files(l_sdisttransf,sort-, > tmpsdist)
     else print(l_sdisttransf, > tmpsdist)
     count(tmpsdist) | scan(nsdist)
     if(nsdist!=1 && nsdist!=nin) {
       printlog("ERROR - NSTRANSFORM: Number of S-distortion calibration \
               files is "//nsdist//". It must be either 1",l_logfile,verbose+)
       printlog("                     or equal to the number of input \
               spectra (inimages).",l_logfile,verbose+)
       delete(tmpsdist,verify-)
       status=1
       goto clean
     }
     
     scanfile=tmpsdist
     while(fscan(scanfile,img) != EOF) {
       sec = "["//l_sci_ext//"]"
       if(!access(s_database//"id"//img//sec)) {
         printlog("ERROR - NSTRANSFORM: s-distortion calibration \
            file "//s_database//img//sec//" does not exist.",l_logfile,verbose+)
         status=1
       }               
     }
     if(status==1) goto clean

} else l_fl_sdisttransf=no

if(!l_fl_lamptransf && !l_fl_sdisttransf) {
  printlog("ERROR - NSTRANSFORM: No wavelength or S-distortion \
     transformations defined",l_logfile,verbose+)
  status=1
  goto clean
}

# At this point tmplamps contains the wavelength calibration files and
#               tmpsdist contains the s-distortion calibration files.

#----------------------------------------------------------------------
# Add input, output lamps and sdist to the input list, matched with the
# correct images
join(tmpinlist,tmpoutlist,delim=" ",missing="Missing",
   maxchar=161,shortest=yes,verbose=no, > tmpin)
delete(tmpinlist//","//tmpoutlist,ver-, >& "dev$null")

if(l_fl_lamptransf) {
  if(nlamps==1) {
    scanfile=tmpin
    while(fscan(scanfile,l_date)!=EOF) {
      print(l_date//" "//l_lamptransf, >> tmpinlist)     
    }
  } else {
    join(tmpin,tmplamps,output=tmpinlist,delim=" ",missing="Missing",
       maxchar=161,shortest+,verbose-)
  } 
} else {
  scanfile=tmpin
  while(fscan(scanfile,l_date)!=EOF) {
    print(l_date//" none ", >> tmpinlist)     
  }
}
delete(tmpin,ver-, >& "dev$null")
rename(tmpinlist,tmpin,field="all")

if(l_fl_sdisttransf) {
  if(nsdist==1) {
    scanfile=tmpin
    while(fscan(scanfile,l_date)!=EOF) {
      print(l_date//" "//l_sdisttransf, >> tmpinlist)     
    }
  } else {
    join(tmpin,tmpsdist,output=tmpinlist,delim=" ",missing="Missing",
       maxchar=161,shortest+,verbose-)
  } 
} else {
  scanfile=tmpin
  while(fscan(scanfile,l_date)!=EOF) {
    print(l_date//" none ", >> tmpinlist)     
  }
}
delete(tmpin,ver-, >& "dev$null")
rename(tmpinlist,tmpin,field="all")
delete(tmplamps//","//tmpsdist,verify-, >>& "dev$null")

#----------------------------------------------------------------------
#  MAIN LOOP for transformation

# First prepare some variables:
print(l_key_dispaxis) | scan(l_key_dispaxis)
lcount = 0
scanfile=tmpin
while(fscan(scanfile,imgin,imgout,lamp,sdist) != EOF ) {

  sec = "["//l_sci_ext//"]"
  lamp = lamp//sec; sdist = sdist//sec
  imgets(imgin//sec,l_key_dispaxis, >& "dev$null")
  if(imgets.value!="0") l_dispaxis=int(imgets.value)
  else l_dispaxis=dispaxis

  longslit.dispaxis=l_dispaxis
  # Fit wavelength solution to input image
  l_fitname_lamp=""
  if(l_fl_lamptransf) {
    fitcoords(lamp,fitname=imgout//"lamp",interactive=l_fl_inter,
       combine=yes,database=l_database,deletions="",
       function=l_function,xorder=l_xorder,yorder=l_yorder,
       logfiles=l_logfile,plotfile="")
    l_fitname_lamp=imgout//"lamp"
  } 

  # Fit the s-distortion solution to input image
  l_fitname_sdist=""
  if(l_fl_sdisttransf) {
    fitcoords(sdist,fitname=imgout//"sdist",
       interactive=l_fl_inter,combine=yes,database=l_database,deletions="",
       function=l_function,xorder=l_xorder,yorder=l_yorder,
       logfiles=l_logfile,plotfile="")
    l_fitname_sdist=imgout//"sdist"
  }

  printlog("Fitnames    : "//l_fitname_lamp//" "//l_fitname_sdist,
     l_logfile,l_verbose)
  l_fitname = l_fitname_lamp//","//l_fitname_sdist
     
  # Transform
  #hedit(imgin//sec,"DISPAXIS",dispaxis,add+,addonly-,dele-,veri-,show-,upda+)
  imcopy(imgin//"[0]",imgout,verbose-)
  transform(imgin//sec,imgout//"["//l_sci_ext//",append]",fitnames=l_fitname,
     database=l_database,interptype=l_interptype,xlog=l_xlog,ylog=l_ylog,
     flux=yes,logfiles=l_logfile) 

  if(imaccess(imgin//"["//l_var_ext//"]")) {
    sec = "["//l_var_ext//"]"
    #hedit(imgin//sec,"DISPAXIS",dispaxis,add+,addonly-,dele-,veri-,show-,upd+)
    transform(imgin//sec,imgout//"["//l_var_ext//",append]",fitnames=l_fitname,
       database=l_database,interptype=l_interptype,xlog=l_xlog,ylog=l_ylog,
       flux=yes,logfiles=l_logfile)
  }
  if(imaccess(imgin//"["//l_dq_ext//"]")) {                 
    sec = "["//l_dq_ext//"]"
    #hedit(imgin//sec,"DISPAXIS",dispaxis,add+,addonly-,del-,veri-,show-,upda+)
    tmpdqint = mktemp("tmpdqintimg")
    transform(imgin//sec,tmpdqint,fitnames=l_fitname,
       database=l_database,interptype=l_interptype,xlog=l_xlog,
       ylog=l_ylog,flux=yes,logfiles=l_logfile)
    imcalc(tmpdqint,imgout//"["//l_dq_ext//",append]","int(im1+0.5)",
       verbose-,pixtype="real")
    imdelete(tmpdqint,verify-)               
  }  
     
  # Update the header
  date | scan(l_date)
  flpr
  gemhedit(imgout//"[0]","NSTRANSF",l_date,"Time stamp for NSTRANSFORM")
  hedit(imgout//"[0]","GEM-TLM",l_date,add+,addonly-,del-,ver-,show-,update+)
                   
} # End of while loop     


#----------------------------------------------------------------------
# Clean up
clean:
if(status==0) {
  printlog("",l_logfile,no)
  printlog("NSTRANSFORM  Exit status good",l_logfile,l_verbose)
  printlog("-------------------------------------------------------------\
          ------------------",l_logfile,l_verbose)
}
scanfile=""
delete(tmpin,ver-, >& "dev$null")
delete(tmpinlist,ver-, >& "dev$null")
delete(tmpoutlist,ver-, >& "dev$null")
delete(tmpfile1,ver-, >& "dev$null")
delete(tmplamps//","//tmpsdist,verify-, >>& "dev$null")

end
