# Copyright(c) 2001-2003 Association of Universities for Research in Astronomy, Inc.

procedure nswavelength(lampspectra)

# Wavelength calibration of NIRI arclamp spectra
#
# Version: Sept 20, 2002 JJ v1.4 release
#          Aug 19, 2003  KL IRAF2.12 - new parameter, addonly, in hedit
#
# Planned: automatic linelist selection based on header keyword;
#          take out nsappwave call if added to nscut for all spectra;

char  lampspectra  {prompt="Input NIRI lamp spectra"}
char  outspectra   {"",prompt="Output spectra"}
char  outprefix    {"w",prompt="Prefix for output spectra"}
real  crval        {INDEF,prompt="Approximate wavelength at coordinate \
                    reference pixel"}
real  cdelt        {INDEF,prompt="Approximate dispersion"}
real  crpix        {INDEF,prompt="Coordinate reference pixel"}
char  key_dispaxis {"DISPAXIS",prompt="Header keywordd for dispersion axis"}
int   dispaxis     {1,min=1,max=2,prompt="Dispersion axis"}

char  database     {"database",prompt="Directory for files containing feature \
                    data"}
char  coordlist    {"gcal$linelists/argon.dat",prompt="User coordinate list, \
                    line list"}
bool  fl_inter     {no,prompt="Examine identifications interactively"}

char  section      {"default",prompt="Image section for running identify"}
int   nsum         {10,min=1,prompt="Number of lines or columns to sum"}
char  ftype        {"emission",min="emission|absorption",prompt="Feature type"}
real  fwidth       {4.,min=2,prompt="Feature width in pixels"}
real  cradius      {5.,min=2,prompt="Centering radius in pixels"}
real  threshold    {0.,prompt="Feature threshold for centering"}
real  minsep       {2.,prompt="Minimum pixel separation for features"}
real  match        {-6.,prompt="Coordinate list matching limit, <0 pixels, >0 \
                    user"}
char  function     {"chebyshev",min="legendre|chebyshev|spline1|spline3",\
                    prompt="Coordinate fitting function"}
int   order        {4,min=1,prompt="Order of coordinate fitting function"}
char  sample       {"*",prompt="Coordinate sample regions"}
int   niterate     {10,min=0,prompt="Rejection iterations"}
real  low_reject   {3.,min=0,prompt="Lower rejection sigma"}
real  high_reject  {3.,min=0,prompt="Upper rejection sigma"}
real  grow         {0.,min=0,prompt="Rejection growing radius"}
bool  refit        {yes,prompt="Refit coordinate function when running \
                    reidentify"}
int   step         {10,prompt="Steps in lines or columns for reidentification"}
bool  trace        {no,prompt="Use fit from previous step rather than central \
                    aperture"}
int   nlost        {3,min=0,prompt="Maximum number of lost features"}
char  aiddebug     {"",prompt="Debug parameter for aidpars"}
char  key_filter   {"FILTER",prompt="Header keyword for filter id"}
char  key_fpmask   {"FPMASK",prompt="Header keyword for focal plane mask"}
char  key_camera   {"CAMERA",prompt="Keyword for filter"}
char  nsappwavedb  {"niri$nsappwave.dat", prompt="Database for nsappwave"}
char  sci_ext      {"SCI",prompt="Name or number of science extension"}
bool  fl_overwrite {yes,prompt="Overwrite existing database entries"}
char  logfile      {"",prompt="Logfile"}
bool  verbose      {yes,prompt="Verbose"}
int   status       {0,prompt="Exit status (0=Good)"}
struct* scanfile   {"",prompt="Internal use"}

begin
char  l_outspectra, l_prefix, l_key_dispaxis, l_database
char  l_coordlist, l_section, l_ftype, l_function, l_sample, l_aiddebug
char  l_key_filter, l_key_fpmask, l_key_camera, l_nsappwavedb
char  l_sci_ext, l_logfile
real  l_crval, l_cdelt, l_crpix, l_fwidth, l_cradius, l_threshold, l_minsep
real  l_match ,l_low_reject, l_high_reject, l_grow
int   l_dispaxis, l_nsum, l_order, l_niterate, l_step, l_nlost
bool  l_fl_inter, l_refit, l_trace, l_fl_overwrite, l_verbose

char  tmpinlist, tmpoutlist, tmpin, s_inter 
char  tmpfile1, l_inimages, l_temp, img
char  imgin, imgout, sec, unixsec
int   nout, nin, nbad, nfiles, count
bool  fl_runbefore
struct message, l_date  

#----------------------------------------------------------------------
# Define temporary files
tmpin = mktemp("tmpin")
tmpinlist = mktemp("tmpinl")
tmpoutlist = mktemp("tmpout")
tmpfile1 = mktemp("tmpfile1")

# Initialize paramters
status=0

# cache parameter files
cache("imgets","niri","nsappwave","nswavelength")

# Set the local variables
l_inimages = lampspectra
l_outspectra=outspectra ; l_prefix=outprefix
l_dispaxis=dispaxis ; l_key_dispaxis=key_dispaxis
l_section=section ; l_database=database;l_coordlist=coordlist 
l_fl_inter=fl_inter ; l_nsum=nsum
l_ftype=ftype ; l_fwidth=fwidth ; l_cradius=cradius ; l_threshold=threshold
l_minsep=minsep ; l_match=match ; l_function=function ; l_order=order
l_sample=sample ; l_niterate=niterate
l_low_reject=low_reject ; l_high_reject=high_reject ; l_grow=grow
l_refit=refit ; l_step=step ; l_trace=trace ; l_nlost=nlost
l_key_filter=key_filter ; l_key_fpmask=key_fpmask; l_key_camera = key_camera
l_nsappwavedb=nsappwavedb 
l_sci_ext=sci_ext
l_fl_overwrite=fl_overwrite ; l_aiddebug=aiddebug
l_logfile=logfile ; l_verbose=verbose 

# Get something in l_crval and l_cdelt even if it is overwritten later
if(crval!=INDEF)
  l_crval=crval
else
  l_crval=1. 
if(cdelt!=INDEF)
  l_cdelt=cdelt
else
  l_cdelt=0.001

print(l_section) | scan(l_section)
if((l_section=="default" || l_section=="") && l_dispaxis==1)
  l_section="middle line"
if((l_section=="default" || l_section=="") && l_dispaxis==2)
  l_section="middle column"

if(!l_verbose) l_aiddebug=""

print(l_logfile) | scan(l_logfile)
if(l_logfile=="") {
   l_logfile=niri.logfile
   if(l_logfile=="") {
      l_logfile="niri.log"
      printlog("WARNING - NSWAVELENGTH:  Both nswavelength.logfile and \
          niri.logfile",logfile=l_logfile, verbose+)
      printlog("                        are empty.  Using niri.log.",
         logfile=l_logfile, verbose+)
   }
}

s_inter=str(l_fl_inter)
if(!l_fl_inter) {
  s_inter="NO"
}

# Write to the logfile
printlog("-------------------------------------------------------------\
     ------------------",l_logfile,l_verbose)
date | scan(l_date)
printlog("NSWAVELENGTH -- "//l_date,l_logfile,l_verbose)
printlog("",l_logfile,l_verbose)

#----------------------------------------------------------------------
# Check access to coordinate file
print(l_coordlist) | scan(l_coordlist)
if(l_coordlist=="") {
  printlog("ERROR - NSWAVELENGTH: Coordinate list not defined",l_logfile,verbose+)
  status=1
  bye
}
if(!access(l_coordlist) ){
  printlog("ERROR - NSWAVELENGTH: Coordinate list "//l_coordlist//" not found",l_logfile,verbose+)
  status=1
  bye
}

#----------------------------------------------------------------------
# Make lists of input spectra
# Put all input images in a temporary file: tmpfile1

if(substr(l_inimages,1,1)=="@") {
  l_temp=substr(l_inimages,2,strlen(l_inimages))
  if(!access(l_temp)) {
    printlog("ERROR - NSWAVELENGTH:  Input file "//l_temp//" not found.",
       l_logfile,verbose+)
    status=1
    goto clean
  }
}

files(l_inimages,sort-, > tmpfile1)

nbad=0; nfiles=0
scanfile = tmpfile1
while(fscan(scanfile,img) != EOF) {
  fl_runbefore=no
  gimverify(img); img=gimverify.outname
  if(gimverify.status==1) {
    printlog("ERROR - NSWAVELENGTH: File "//img//" not found.",l_logfile,verbose+)
    nbad+=1
  } else if(gimverify.status>1) {
    printlog("ERROR - NSWAVELENGTH: File "//img//" not a MEF FITS image.",
       l_logfile,verbose+)
    nbad+=1
  } else {
    imgets(img//"[0]","NPREPARE", >& "dev$null")
    if (imgets.value == "0") {
      printlog("ERROR - NSWAVELENGTH: Image "//img//" not NPREPAREd.",
         l_logfile,verbose+)
      nbad+=1
    }
    imgets(img//"[0]","NSWAVELE", >& "dev$null")
    if (imgets.value != "0") {
      printlog("WARNING - NSWAVELENGTH: Image "//img//" has been run",l_logfile,verbose+)
      printlog("                        through nswavelength before.",l_logfile,verbose+)
      fl_runbefore=yes
    }                             
  }

  if(!fl_runbefore) {
    nfiles+=1
    print(img, >> tmpinlist)
  }      
} # end while loop
if(access(tmpinlist)) {
  count(tmpinlist) | scan(nfiles)
}

# Check for empty file list
if(nfiles==0) {
  printlog("ERROR - NSWAVELENGTH: No input images to process.",
     l_logfile,verbose+)
  status=1
  goto clean
}

# Exit if problems found with input files
if (nbad > 0) {
  printlog("ERROR - NSWAVELENGTH: "//nbad//" image(s) either do not exist, \
    are not MEF files, or",l_logfile, verbose+)
  printlog("                 have not been run through NPREPARE.",l_logfile,verbose+)
  status=1
  goto clean
} # end if (nbad > 0)

count(tmpinlist) | scan(nin)

printlog("Using input files:",l_logfile,l_verbose)
if(l_verbose) type(tmpinlist)
type(tmpinlist, >> l_logfile)
delete(tmpfile1, ver-)

#----------------------------------------------------------------------
# Now for the output images

if(substr(l_outspectra,1,1)=="@") {
  l_temp=substr(l_outspectra,2,strlen(l_outspectra))
  if(!access(l_temp)) {
    printlog("ERROR - NSWAVELENGTH:  Input file "//l_temp//" not found.",
     l_logfile,verbose+)
    status=1
    goto clean
  }
}

files(l_outspectra,sort-, > tmpfile1)

# If tmpfile1 is empty, the output files names should be created with the
# prefix parameter
count=0; count(tmpfile1) | scan(count)
if(count==0) {
  delete(tmpfile1,ver-)
  scanfile=tmpinlist
  while(fscan(scanfile,img) != EOF) {
    print(l_prefix//img, >> tmpfile1)
  }
}               

# Verify that the output images do not exist
nbad=0
nfiles=0
scanfile = tmpfile1
while(fscan(scanfile,img) != EOF) {
  gimverify(img)
  img = gimverify.outname
  if(gimverify.status!=1) {
    printlog("ERROR - NSWAVELENGTH: File "//img//" already exists.",l_logfile,
       verbose+)
    nbad+=1
  } 
  nfiles+=1
  print(img, >> tmpoutlist)
} # end while loop

# Exit if problems found with output files
if (nbad > 0) {
  status=1
  goto clean
} # end if (nbad > 0)

count(tmpoutlist) | scan(nout)

printlog("Using output files:",l_logfile,l_verbose)
if(l_verbose) type(tmpoutlist)
type(tmpoutlist, >> l_logfile)
delete(tmpfile1, ver-)

# Check number of input output images
if(nin!=nout) {
  printlog("ERROR - NSWAVELENGTH: Different number of input and output \
     spectra",l_logfile,verbose+)
  status=1
  goto clean
}

# make the file list
join(tmpinlist,tmpoutlist,output=tmpin,delim=" ",missing=" ",
   maxchars=161,shortest+,verbose-)

#-------------------------------------------------------------------------
#  MAIN LOOP

scanfile=tmpin
while(fscan(scanfile,imgin,imgout) != EOF) {

  sec = "["//l_sci_ext//"]"
  printlog("Lamp image: "//imgin//sec,l_logfile,l_verbose)
  
  # Run nsappwave if necessary          
  imgets(imgin//"[0]","NSAPPWAV", >& "dev$null")
  if(imgets.value=="0") {
    nsappwave(imgin,outspectra=imgout,prefix="",
       nsappwavedb=l_nsappwavedb,crval=l_crval,cdelt=l_cdelt,
       key_filter=l_key_filter,key_fpmask=l_key_fpmask,
       key_camera=l_key_camera,key_dispaxis=l_key_dispaxis,
       sci_ext=l_sci_ext,logfile=l_logfile,verbose=l_verbose)

    if(nsappwave.status>0) {
      printlog("ERROR - NSWAVELENGTH: nsappwave returned with error",l_logfile,verbose+)
      status=1
      goto clean
    } 
  } else copy(imgin//".fits",imgout//".fits",ver+)              
 
  # Get the approximate wavelength info from the header
  imgets(imgout//sec,"CRPIX"//l_dispaxis) 
  l_crpix=real(imgets.value)
  imgets(imgout//sec,"CRVAL"//l_dispaxis) 
  l_crval=real(imgets.value)
  imgets(imgout//sec,"CD"//l_dispaxis//"_"//l_dispaxis) 
  l_cdelt=real(imgets.value)
          
  longslit.dispaxis=l_dispaxis

  autoidentify(imgout//sec,l_crval,l_cdelt,coordlist=l_coordlist,
     units="Angstrom",interactive=s_inter,aidpars="",
     section=l_section,nsum=l_nsum,ftype=l_ftype, fwidth=l_fwidth,
     cradius=l_cradius,threshold=l_threshold,minsep=l_minsep,   
     match=l_match,function=l_function,order=l_order,
     sample=l_sample,niterate=l_niterate,low_reject=l_low_reject,
     high_reject=l_high_reject,grow=l_grow,dbwrite="YES",
     overwrite=l_fl_overwrite,database=l_database,verbose=l_verbose,
     logfile=l_logfile,plotfile="",graphics="stdgraph",cursor="",
     aidpars.debug=l_aiddebug,aidpars.cddir="sign",aidpars.crpix=l_crpix)

  reidentify(imgout//sec,imgout//sec,coordli=l_coordlist,
     interactive=s_inter,section=l_section,newaps=yes,
     refit=l_refit,trace=l_trace,step=l_step,nsum=l_nsum,shift="0.",
     search=0.,nlost=l_nlost,cradius=l_cradius,threshold=l_threshold,
     addfeatures=no,match=l_match,maxfeatures=250,minsep=l_minsep,
     override=l_fl_overwrite,database=l_database,verbose=l_verbose,
     logfile=l_logfile,plotfile="",graphics="stdgraph",cursor="")

  # update header
  gemhedit(imgout//"[0]","NSWAVELE",l_date,"Time stamp for NSWAVELENGTH")
  hedit(imgout//"[0]","GEM-TLM",l_date,add-,addonly-,delete-,ver-,show-,update+)

  # Rename the idfile so fitcoords can find it
  # sec is the extension name, e.g. [SCI]
  if(l_fl_overwrite) {
    # Using UNIX rm and mv commands because cl delete and rename
    # do not allow file names with [] (e.g. a[x,y]):
    delete(".nswave.csh",ver-, >& "dev$null")
    unixsec = "\["//l_sci_ext//"\]"
    print("rm "//l_database//"/id"//imgout//unixsec, > \
       ".nswave.csh")
    print("mv "//l_database//"/id"//imgout//" "//\
       l_database//"/id"//imgout//unixsec, >> ".nswave.csh")

    !chmod +x .nswave.csh                    
    !.nswave.csh >& .nswave.log
    delete(".nswave.csh,.nswave.log",ver-, >& "dev$null")
  } else {
    type(l_database//"/id"//imgout, >> l_database//"/id"//imgout//sec)
    delete(l_database//"/id"//imgout,verify-, >& "dev$null")
  }    
} # end while     

#----------------------------------------------------------------------
# Clean up
clean:
scanfile=""
delete(tmpin,ver-, >& "dev$null")
delete(tmpinlist,ver-, >& "dev$null")
delete(tmpoutlist,ver-, >& "dev$null")
delete(tmpfile1,ver-, >& "dev$null")

printlog("",l_logfile,l_verbose)
if(status==0)
  printlog("NSWAVELENGTH Exit status good",l_logfile,l_verbose)
  printlog("---------------------------------------------------------------\
     ----------------",l_logfile,l_verbose)

end

